 /* msidefs.cpy   Windows Installer data definitions                  */
 /*               1999 Microsoft Corporation                         */
 /* __________________________________________________________________*/

 %dcl #_MSIDEFS char ext;
 %if #_MSIDEFS = '' %then %do;
 %#_MSIDEFS = 'Y';

 %dcl #WINDEF_ char ext;
 %if #WINDEF_ = '' %then %do;
 %include windef;
 %end;

 /* _________________________________________________________________ */
 /*                                                                   */
 /* INSTALLER PROPERTY DEFINITIONS                                    */
 /* _________________________________________________________________ */

 /* Required properties: set in Property table */
 dcl IPROPNAME_PRODUCTNAME         TCHAR value('ProductName');
     /* name registered for display */
 dcl IPROPNAME_PRODUCTCODE         TCHAR value('ProductCode');
     /* unique string GUID for product */
 dcl IPROPNAME_PRODUCTVERSION      TCHAR value('ProductVersion');
     /* string product version */
 dcl IPROPNAME_INSTALLLANGUAGE     TCHAR value('ProductLanguage');
     /* install language of product, use to load resources */
 dcl IPROPNAME_MANUFACTURER        TCHAR value('Manufacturer');
     /* name of manufacturer */

 /* Customization properties: set in Property table */
 dcl IPROPNAME_UPGRADECODE         TCHAR value('UpgradeCode');
     /* unique string GUID for product family */
 dcl IPROPNAME_PIDTEMPLATE         TCHAR value('PIDTemplate');
     /* drives Product ID processing */
 dcl IPROPNAME_DISKPROMPT          TCHAR value('DiskPrompt');
     /* prompt for CD */
 dcl IPROPNAME_LEFTUNIT            TCHAR value('LeftUnit');
     /* name of unit placed to left of number instead of right */
 dcl IPROPNAME_ADMIN_PROPERTIES    TCHAR value('AdminProperties');
     /* properties to stuff in admin package */
 dcl IPROPNAME_DEFAULTUIFONT       TCHAR value('DefaultUIFont');
     /* the font used in the UI if no other font is specified */
 dcl IPROPNAME_ALLOWEDPROPERTIES   TCHAR
     value('SecureCustomProperties');
 dcl IPROPNAME_ENABLEUSERCONTROL   TCHAR value('EnableUserControl');
     /* allows user to specify any public property */

 /* Customization properties: set on command-line or in Property table*/
 dcl IPROPNAME_USERNAME            TCHAR value('USERNAME');
 dcl IPROPNAME_COMPANYNAME         TCHAR value('COMPANYNAME');
 dcl IPROPNAME_PIDKEY              TCHAR value('PIDKEY');
     /* used with PIDTemplate to form ProductID */
 dcl IPROPNAME_PATCH               TCHAR value('PATCH');
     /* patch package to apply - SET BY INSTALLER */
 dcl IPROPNAME_TARGETDIR           TCHAR value('TARGETDIR');
     /* target location - defaults to ROOTDRIVE */
 dcl IPROPNAME_ACTION              TCHAR value('ACTION');
     /* top-level action to perform - default to INSTALL */
 dcl IPROPNAME_LIMITUI             TCHAR value('LIMITUI');
     /* limit ui level to Basic */
 dcl IPROPNAME_LOGACTION           TCHAR value('LOGACTION');
     /* log only these actions */
 dcl IPROPNAME_ALLUSERS            TCHAR value('ALLUSERS');
     /* install for all users */
 dcl IPROPNAME_INSTALLLEVEL        TCHAR value('INSTALLLEVEL');
 dcl IPROPNAME_REBOOT              TCHAR value('REBOOT');
     /* force or suppress reboot */
 dcl IPROPNAME_EXECUTEMODE         TCHAR value('EXECUTEMODE');
     /* NONE or SCRIPT */
 dcl IPROPVALUE_EXECUTEMODE_NONE   TCHAR value('NONE');
     /* do not update system */
 dcl IPROPVALUE_EXECUTEMODE_SCRIPT TCHAR value('SCRIPT');
     /* default - run script to update system */
 dcl IPROPNAME_EXECUTEACTION       TCHAR value('EXECUTEACTION');
     /* run action on server side */
 dcl IPROPNAME_SOURCELIST          TCHAR value('SOURCELIST');
 dcl IPROPNAME_ROOTDRIVE           TCHAR value('ROOTDRIVE');
     /* default drive to install - SET BY INSTALLER */
 dcl IPROPNAME_TRANSFORMS          TCHAR value('TRANSFORMS');
     /* transforms to apply */
 dcl IPROPNAME_TRANSFORMSATSOURCE  TCHAR value('TRANSFORMSATSOURCE');
     /* transforms can be found at source */
 dcl IPROPNAME_TRANSFORMSSECURE    TCHAR value('TRANSFORMSSECURE');
     /* file transforms are secured */
 dcl IPROPNAME_SEQUENCE            TCHAR value('SEQUENCE');
     /* sequence table to run with SEQUENCE action */
 dcl IPROPNAME_SHORTFILENAMES      TCHAR value('SHORTFILENAMES');
     /* force short file names */
 dcl IPROPNAME_PRIMARYFOLDER       TCHAR value('PRIMARYFOLDER');
     /* Folder on the volume the author wants costing info for */
 dcl IPROPNAME_AFTERREBOOT         TCHAR value('AFTERREBOOT');
     /* install is after a ForceReboot triggered reboot */
 dcl IPROPNAME_NOCOMPANYNAME       TCHAR value('NOCOMPANYNAME');
 dcl IPROPNAME_NOUSERNAME          TCHAR value('NOUSERNAME');
 dcl IPROPNAME_DISABLEROLLBACK     TCHAR value('DISABLEROLLBACK');
     /* disable rollback for this install */
 dcl IPROPNAME_AVAILABLEFREEREG    TCHAR value('AVAILABLEFREEREG');
     /* set up the free space in the registry before commencing */
     /* the install */
 dcl IPROPNAME_DISABLEADVTSHORTCUTS TCHAR
     value('DISABLEADVTSHORTCUTS');
     /* disable creating darwin shortcuts even if supported */
 dcl IPROPNAME_PATCHNEWPACKAGECODE TCHAR value('PATCHNEWPACKAGECODE');
     /* added to property table by patch transforms - used to update */
     /* PackageCode of admin packages when patching admin installs */
 dcl IPROPNAME_PATCHNEWSUMMARYSUBJECT TCHAR
     value('PATCHNEWSUMMARYSUBJECT');
     /* added to property table by patch transforms - used to update */
     /* Subject summary info property of admin packages when patching*/
     /* admin installs */
 dcl IPROPNAME_PATCHNEWSUMMARYCOMMENTS TCHAR
     value('PATCHNEWSUMMARYCOMMENTS');
     /* added to property table by patch transforms - used to update */
     /* Comments summary info property of admin packages when patching*/
     /* admin installs */
 dcl IPROPNAME_PRODUCTLANGUAGE     TCHAR value('PRODUCTLANGUAGE');
     /* requested language, must be one in summary information list, */
     /* selects language transform */

 /* Properties used to populate Add/Remove Control Panel values */
 dcl IPROPNAME_ARPAUTHORIZEDCDFPREFIX TCHAR
     value('ARPAUTHORIZEDCDFPREFIX');
 dcl IPROPNAME_ARPCOMMENTS         TCHAR value('ARPCOMMENTS');
 dcl IPROPNAME_ARPCONTACT          TCHAR value('ARPCONTACT');
 dcl IPROPNAME_ARPHELPLINK         TCHAR value('ARPHELPLINK');
 dcl IPROPNAME_ARPHELPTELEPHONE    TCHAR value('ARPHELPTELEPHONE');
 dcl IPROPNAME_ARPINSTALLLOCATION  TCHAR value('ARPINSTALLLOCATION');
 dcl IPROPNAME_ARPNOMODIFY         TCHAR value('ARPNOMODIFY');
 dcl IPROPNAME_ARPNOREMOVE         TCHAR value('ARPNOREMOVE');
 dcl IPROPNAME_ARPNOREPAIR         TCHAR value('ARPNOREPAIR');
 dcl IPROPNAME_ARPREADME           TCHAR value('ARPREADME');
 dcl IPROPNAME_ARPSIZE             TCHAR value('ARPSIZE');
 dcl IPROPNAME_ARPSYSTEMCOMPONENT  TCHAR value('ARPSYSTEMCOMPONENT');
 dcl IPROPNAME_ARPURLINFOABOUT     TCHAR value('ARPURLINFOABOUT');
 dcl IPROPNAME_ARPURLUPDATEINFO    TCHAR value('ARPURLUPDATEINFO');

 /* Dynamic properties set by installer during install */
 dcl IPROPNAME_INSTALLED           TCHAR value('Installed');
     /* product already installed */
 dcl IPROPNAME_PRODUCTSTATE        TCHAR value('ProductState');
     /* state of product (installed,advertised,etc...) */
 dcl IPROPNAME_PRESELECTED         TCHAR value('Preselected');
     /* selections made on command line */
 dcl IPROPNAME_RESUME              TCHAR value('RESUME');
     /* resuming suspended install */
 dcl IPROPNAME_UPDATESTARTED       TCHAR value('UpdateStarted');
     /* have begun to update system */
 dcl IPROPNAME_PRODUCTID           TCHAR value('ProductID');
     /* the complete validated Product ID */
 dcl IPROPNAME_OUTOFDISKSPACE      TCHAR value('OutOfDiskSpace');
 dcl IPROPNAME_OUTOFNORBDISKSPACE  TCHAR value('OutOfNoRbDiskSpace');
 dcl IPROPNAME_COSTINGCOMPLETE     TCHAR value('CostingComplete');
 dcl IPROPNAME_SOURCEDIR           TCHAR value('SourceDir');
     /* source location - SET BY INSTALLER */
 dcl IPROPNAME_REPLACEDINUSEFILES  TCHAR value('ReplacedInUseFiles');
     /* need reboot to completely install one or more files */
 dcl IPROPNAME_PRIMARYFOLDER_PATH  TCHAR value('PrimaryVolumePath');
 dcl IPROPNAME_PRIMARYFOLDER_SPACEAVAILABLE TCHAR
     value('PrimaryVolumeSpaceAvailable');
 dcl IPROPNAME_PRIMARYFOLDER_SPACEREQUIRED TCHAR
     value('PrimaryVolumeSpaceRequired');
 dcl IPROPNAME_PRIMARYFOLDER_SPACEREMAINING TCHAR
     value('PrimaryVolumeSpaceRemaining');
 dcl IPROPNAME_ISADMINPACKAGE      TCHAR value('IsAdminPackage');
 dcl IPROPNAME_ROLLBACKDISABLED    TCHAR value('RollbackDisabled');
 dcl IPROPNAME_RESTRICTEDUSERCONTROL TCHAR
     value('RestrictedUserControl');

 /* Dynamic properties evaluated upon use */
 dcl IPROPNAME_TIME                TCHAR value('Time');
 dcl IPROPNAME_DATE                TCHAR value('Date');
 dcl IPROPNAME_DATETIME            TCHAR value('DateTime');

 /* Hardware properties: set by installer at initialization */
 dcl IPROPNAME_INTEL               TCHAR value('Intel');
 dcl IPROPNAME_ALPHA               TCHAR value('Alpha');
 dcl IPROPNAME_TEXTHEIGHT          TCHAR value('TextHeight');
 dcl IPROPNAME_SCREENX             TCHAR value('ScreenX');
 dcl IPROPNAME_SCREENY             TCHAR value('ScreenY');
 dcl IPROPNAME_CAPTIONHEIGHT       TCHAR value('CaptionHeight');
 dcl IPROPNAME_BORDERTOP           TCHAR value('BorderTop');
 dcl IPROPNAME_BORDERSIDE          TCHAR value('BorderSide');
 dcl IPROPNAME_COLORBITS           TCHAR value('ColorBits');
 dcl IPROPNAME_PHYSICALMEMORY      TCHAR value('PhysicalMemory');
 dcl IPROPNAME_VIRTUALMEMORY       TCHAR value('VirtualMemory');

 /* Operating System properties: set by installer at initialization */
 dcl IPROPNAME_VERSIONNT           TCHAR value('VersionNT');
 dcl IPROPNAME_VERSION9X           TCHAR value('Version9X');
 dcl IPROPNAME_WINDOWSBUILD        TCHAR value('WindowsBuild');
 dcl IPROPNAME_SERVICEPACKLEVEL    TCHAR value('ServicePackLevel');
 dcl IPROPNAME_SHAREDWINDOWS       TCHAR value('SharedWindows');
 dcl IPROPNAME_COMPUTERNAME        TCHAR value('ComputerName');
 dcl IPROPNAME_SHELLADVTSUPPORT    TCHAR value('ShellAdvtSupport');
 dcl IPROPNAME_OLEADVTSUPPORT      TCHAR value('OLEAdvtSupport');
 dcl IPROPNAME_SYSTEMLANGUAGEID    TCHAR value('SystemLanguageID');
 dcl IPROPNAME_TTCSUPPORT          TCHAR value('TTCSupport');
 dcl IPROPNAME_TERMSERVER          TCHAR value('TerminalServer');

 /* User properties: set by installer at initialization */
 dcl IPROPNAME_LOGONUSER           TCHAR value('LogonUser');
 dcl IPROPNAME_USERSID             TCHAR value('UserSID');
 dcl IPROPNAME_ADMINUSER           TCHAR value('AdminUser');
 dcl IPROPNAME_USERLANGUAGEID      TCHAR value('UserLanguageID');
 dcl IPROPNAME_PRIVILEGED          TCHAR value('Privileged');

 /* System folder properties: set by installer at initialization */
 dcl IPROPNAME_WINDOWS_FOLDER      TCHAR value('WindowsFolder');
 dcl IPROPNAME_SYSTEM_FOLDER       TCHAR value('SystemFolder');
 dcl IPROPNAME_SYSTEM16_FOLDER     TCHAR value('System16Folder');
 dcl IPROPNAME_WINDOWS_VOLUME      TCHAR value('WindowsVolume');
 dcl IPROPNAME_TEMP_FOLDER         TCHAR value('TempFolder');
 dcl IPROPNAME_PROGRAMFILES_FOLDER TCHAR value('ProgramFilesFolder');
 dcl IPROPNAME_COMMONFILES_FOLDER  TCHAR value('CommonFilesFolder');
 dcl IPROPNAME_STARTMENU_FOLDER    TCHAR value('StartMenuFolder');
 dcl IPROPNAME_PROGRAMMENU_FOLDER  TCHAR value('ProgramMenuFolder');
 dcl IPROPNAME_STARTUP_FOLDER      TCHAR value('StartupFolder');
 dcl IPROPNAME_NETHOOD_FOLDER      TCHAR value('NetHoodFolder');
 dcl IPROPNAME_PERSONAL_FOLDER     TCHAR value('PersonalFolder');
 dcl IPROPNAME_SENDTO_FOLDER       TCHAR value('SendToFolder');
 dcl IPROPNAME_DESKTOP_FOLDER      TCHAR value('DesktopFolder');
 dcl IPROPNAME_TEMPLATE_FOLDER     TCHAR value('TemplateFolder');
 dcl IPROPNAME_FONTS_FOLDER        TCHAR value('FontsFolder');
 dcl IPROPNAME_FAVORITES_FOLDER    TCHAR value('FavoritesFolder');
 dcl IPROPNAME_RECENT_FOLDER       TCHAR value('RecentFolder');
 dcl IPROPNAME_APPDATA_FOLDER      TCHAR value('AppDataFolder');
 dcl IPROPNAME_PRINTHOOD_FOLDER    TCHAR value('PrintHoodFolder');

 /* Feature/Component installation properties: set on command-line */
 dcl IPROPNAME_FEATUREADDLOCAL     TCHAR value('ADDLOCAL');
 dcl IPROPNAME_FEATUREADDSOURCE    TCHAR value('ADDSOURCE');
 dcl IPROPNAME_FEATUREADDDEFAULT   TCHAR value('ADDDEFAULT');
 dcl IPROPNAME_FEATUREREMOVE       TCHAR value('REMOVE');
 dcl IPROPNAME_FEATUREADVERTISE    TCHAR value('ADVERTISE');
 dcl IPROPVALUE_FEATURE_ALL        TCHAR value('ALL');

 dcl IPROPNAME_COMPONENTADDLOCAL   TCHAR value('COMPADDLOCAL');
 dcl IPROPNAME_COMPONENTADDSOURCE  TCHAR value('COMPADDSOURCE');
 dcl IPROPNAME_COMPONENTADDDEFAULT TCHAR value('COMPADDDEFAULT');

 dcl IPROPNAME_FILEADDLOCAL        TCHAR value('FILEADDLOCAL');
 dcl IPROPNAME_FILEADDSOURCE       TCHAR value('FILEADDSOURCE');
 dcl IPROPNAME_FILEADDDEFAULT      TCHAR value('FILEADDDEFAULT');

 dcl IPROPNAME_REINSTALL           TCHAR value('REINSTALL');
 dcl IPROPNAME_REINSTALLMODE       TCHAR value('REINSTALLMODE');
 dcl IPROPNAME_PROMPTROLLBACKCOST  TCHAR value('PROMPTROLLBACKCOST');
 dcl IPROPVALUE_RBCOST_PROMPT      TCHAR value('P');
 dcl IPROPVALUE_RBCOST_SILENT      TCHAR value('D');
 dcl IPROPVALUE_RBCOST_FAIL        TCHAR value('F');

 /* _________________________________________________________________ */
 /*                                                                   */
 /* TOP-LEVEL ACTION NAMES                                            */
 /* _________________________________________________________________ */

 dcl IACTIONNAME_INSTALL           TCHAR value('INSTALL');
 dcl IACTIONNAME_ADVERTISE         TCHAR value('ADVERTISE');
 dcl IACTIONNAME_ADMIN             TCHAR value('ADMIN');
 dcl IACTIONNAME_SEQUENCE          TCHAR value('SEQUENCE');
 dcl IACTIONNAME_COLLECTUSERINFO   TCHAR value('CollectUserInfo');
 dcl IACTIONNAME_FIRSTRUN          TCHAR value('FirstRun');

 /* _________________________________________________________________ */
 /*                                                                   */
 /*  SummaryInformation property stream property IDs                  */
 /* _________________________________________________________________ */

 /* PID_SECURITY defined as ('80000002'xu) in objidl.cpy, */
 /* need to redefine here                                 */

 /* standard property definitions, from OLE2 documentation */
 dcl PID_DICTIONARY   fixed bin(31) value(0 );
     /* integer count + array of entries */
 dcl PID_CODEPAGE     fixed bin(31) value('1'xu);
     /* short integer */
 dcl PID_TITLE        fixed bin(31) value(2);
     /* string */
 dcl PID_SUBJECT      fixed bin(31) value(3);
     /* string */
 dcl PID_AUTHOR       fixed bin(31) value(4);
     /* string */
 dcl PID_KEYWORDS     fixed bin(31) value(5);
     /* string */
 dcl PID_COMMENTS     fixed bin(31) value(6);
     /* string */
 dcl PID_TEMPLATE     fixed bin(31) value(7);
     /* string */
 dcl PID_LASTAUTHOR   fixed bin(31) value(8);
     /* string */
 dcl PID_REVNUMBER    fixed bin(31) value(9);
     /* string */
 dcl PID_EDITTIME     fixed bin(31) value(10);
     /* datatime */
 dcl PID_LASTPRINTED  fixed bin(31) value(11);
     /* datetime */
 dcl PID_CREATE_DTM   fixed bin(31) value(12);
     /* datetime */
 dcl PID_LASTSAVE_DTM fixed bin(31) value(13);
     /* datetime */
 dcl PID_PAGECOUNT    fixed bin(31) value(14);
     /* integer */
 dcl PID_WORDCOUNT    fixed bin(31) value(15);
     /* integer */
 dcl PID_CHARCOUNT    fixed bin(31) value(16);
     /* integer */
 dcl PID_THUMBNAIL    fixed bin(31) value(17);
     /* clipboard format + metafile/bitmap (not supported) */
 dcl PID_APPNAME      fixed bin(31) value(18);
     /* string */
 %dcl PID_SECURITY char; %PID_SECURITY = '19'; /* see above */
     /* integer */

 /* PIDs given specific meanings for Installer */
 dcl PID_MSIVERSION   fixed bin(31) value(PID_PAGECOUNT);
     /* integer, Installer version number (major*100+minor) */
 dcl PID_MSISOURCE    fixed bin(31) value(PID_WORDCOUNT);
     /* integer, type of file image, short/long, media/tree */
 dcl PID_MSIRESTRICT  fixed bin(31) value(PID_CHARCOUNT);
     /* integer, transform restrictions */

 /* _________________________________________________________________ */
 /*                                                                   */
 /* INSTALLER DATABASE INTEGER COLUMN DEFINITIONS                     */
 /* _________________________________________________________________ */

 /* PL/I Note: Original C uses enums for the following.               */
 /* Using PL/I ordinals would make coding exceedingly difficult as    */
 /* PL/I ordinals are much more strongly typed                        */
 /* (in PL/I they are not interchangeable with integers).             */

 /* BBControl.Attributes */
 /* Control.Attributes */

 define alias msidbControlAttributes             fixed bin(32) unsigned;

 dcl ( msidbControlAttributesVisible                value('00000001'xu),
       msidbControlAttributesEnabled                value('00000002'xu),
       msidbControlAttributesSunken                 value('00000004'xu),
       msidbControlAttributesIndirect               value('00000008'xu),
       msidbControlAttributesInteger                value('00000010'xu),
       msidbControlAttributesRTLRO                  value('00000020'xu),
       msidbControlAttributesRightAligned           value('00000040'xu),
       msidbControlAttributesLeftScroll             value('00000080'xu),
       msidbControlAttributesBiDi                   value(
                  ior (msidbControlAttributesRTLRO,
                       msidbControlAttributesRightAligned,
                       msidbControlAttributesLeftScroll)),

       /* Text controls */
       msidbControlAttributesTransparent            value('00010000'xu),
       msidbControlAttributesNoPrefix               value('00020000'xu),
       msidbControlAttributesNoWrap                 value('00040000'xu),
       msidbControlAttributesFormatSize             value('00080000'xu),
       msidbControlAttributesUsersLanguage          value('00100000'xu),

       /* Edit controls */
       msidbControlAttributesMultiline              value('00010000'xu),

       /* ProgressBar controls */
       msidbControlAttributesProgress95             value('00010000'xu),

       /* VolumeSelectCombo and DirectoryCombo controls */
       msidbControlAttributesRemovableVolume        value('00010000'xu),
       msidbControlAttributesFixedVolume            value('00020000'xu),
       msidbControlAttributesRemoteVolume           value('00040000'xu),
       msidbControlAttributesCDROMVolume            value('00080000'xu),
       msidbControlAttributesRAMDiskVolume          value('00100000'xu),
       msidbControlAttributesFloppyVolume           value('00200000'xu),
       /* VolumeCostList controls */
       msidbControlShowRollbackCost                 value('00400000'xu),

       /* ListBox and ComboBox controls */
       msidbControlAttributesSorted                 value('00010000'xu),
       msidbControlAttributesComboList              value('00020000'xu),

       /* picture button controls */
       msidbControlAttributesImageHandle            value('00010000'xu),
       msidbControlAttributesPushLike               value('00020000'xu),
       msidbControlAttributesBitmap                 value('00040000'xu),
       msidbControlAttributesIcon                   value('00080000'xu),
       msidbControlAttributesFixedSize              value('00100000'xu),
       msidbControlAttributesIconSize16             value('00200000'xu),
       msidbControlAttributesIconSize32             value('00400000'xu),
       msidbControlAttributesIconSize48             value('00600000'xu),

       /* RadioButton controls */
       msidbControlAttributesHasBorder              value('01000000'xu)
   ) type msidbControlAttributes;

 /* CompLocator.Type */
 /* IniLocator.Type */
 /* RegLocator.Type */
 define alias msidbLocatorType                   fixed bin(32) unsigned;
 dcl ( msidbLocatorTypeDirectory                    value('00000000'xu),
       msidbLocatorTypeFileName                     value('00000001'xu)
     ) type msidbLocatorType;

 /* Component.Attributes */
 define alias msidbComponentAttributes           fixed bin(32) unsigned;
 dcl ( msidbComponentAttributesLocalOnly            value('00000000'xu),
       msidbComponentAttributesSourceOnly           value('00000001'xu),
       msidbComponentAttributesOptional             value('00000002'xu),
          /* local or source */
       msidbComponentAttributesRegistryKeyPath      value('00000004'xu),
          /* KeyPath is key to Registry table */
       msidbComponentAttributesSharedDllRefCount    value('00000008'xu),
          /* increment SharedDll count */
       msidbComponentAttributesPermanent            value('00000010'xu),
          /* never uninstall component */
       msidbComponentAttributesODBCDataSource       value('00000020'xu),
          /* KeyFile is key to ODBCDataSource table */
       msidbComponentAttributesTransitive           value('00000040'xu),
          /* Can transition to/from installed/uninstalled based on */
          /* changing conditional */
       msidbComponentAttributesNeverOverwrite       value('00000080'xu)
          /* dont stomp over existing component if key path exists */
          /* (file/ regkey) */
     ) type msidbComponentAttributes;

 /* CustomAction.Type */
 define alias msidbCustomActionType              fixed bin(32) unsigned;
 dcl ( /* executable types */
       msidbCustomActionTypeDll                     value('00000001'xu),
          /* Target = entry point name */
       msidbCustomActionTypeExe                     value('00000002'xu),
          /* Target = command line args */
       msidbCustomActionTypeTextData                value('00000003'xu),
          /* Target = text string to be formatted */
          /* and set into property */
       msidbCustomActionTypeJScript                 value('00000005'xu),
          /* Target = entry point name, null if none to call */
       msidbCustomActionTypeVBScript                value('00000006'xu),
          /* Target = entry point name, null if none to call */
       msidbCustomActionTypeInstall                 value('00000007'xu),
          /* Target = property list for nested engine initialization */

       /* source of code */
       msidbCustomActionTypeBinaryData              value('00000000'xu),
          /* Source = Binary.Name, data stored in stream */
       msidbCustomActionTypeSourceFile              value('00000010'xu),
          /* Source = File.File, file part of installation */
       msidbCustomActionTypeDirectory               value('00000020'xu),
          /* Source = Directory.Directory, folder containing */
          /* existing file */
       msidbCustomActionTypeProperty                value('00000030'xu),
          /* Source = Property.Property, full path to executable */

       /* return processing - default is syncronous execution, */
       /* process return code */
       msidbCustomActionTypeContinue                value('00000040'xu),
          /* ignore action return status, continue running */
       msidbCustomActionTypeAsync                   value('00000080'xu),
          /* run asynchronously */

       /* execution scheduling flags - default is execute
       /* whenever sequenced */
       msidbCustomActionTypeFirstSequence           value('00000100'xu),
          /* skip if UI sequence already run */
       msidbCustomActionTypeOncePerProcess          value('00000200'xu),
          /* skip if UI sequence already run in same process */
       msidbCustomActionTypeClientRepeat            value('00000300'xu),
          /* run on client only if UI already run on client */
       msidbCustomActionTypeInScript                value('00000400'xu),
          /* queue for execution within script */
       msidbCustomActionTypeRollback                value('00000100'xu),
          /* in conjunction with InScript: queue in Rollback script */
       msidbCustomActionTypeCommit                  value('00000200'xu),
          /* in conjunction with InScript: run Commit ops from script */
          /* on success */

       /* security context flag, default to impersonate as user, */
       /* valid only if InScript */
       msidbCustomActionTypeNoImpersonate           value('00000800'xu)
          /* no impersonation, run in system context */
     ) type msidbCustomActionType;

 /* Dialog.Attributes */
 define alias msidbDialogAttributes              fixed bin(32) unsigned;
 dcl ( msidbDialogAttributesVisible                 value('00000001'xu),
       msidbDialogAttributesModal                   value('00000002'xu),
       msidbDialogAttributesMinimize                value('00000004'xu),
       msidbDialogAttributesSysModal                value('00000008'xu),
       msidbDialogAttributesKeepModeless            value('00000010'xu),
       msidbDialogAttributesTrackDiskSpace          value('00000020'xu),
       msidbDialogAttributesUseCustomPalette        value('00000040'xu),
       msidbDialogAttributesRTLRO                   value('00000080'xu),
       msidbDialogAttributesRightAligned            value('00000100'xu),
       msidbDialogAttributesLeftScroll              value('00000200'xu),
       msidbDialogAttributesBiDi                    value(
            ior(msidbDialogAttributesRTLRO,
                msidbDialogAttributesRightAligned,
                msidbDialogAttributesLeftScroll)),
       msidbDialogAttributesError                   value('00010000'xu)
     ) type msidbDialogAttributes;

 /* Feature.Attributes */
 define alias msidbFeatureAttributes             fixed bin(32) unsigned;
 dcl ( msidbFeatureAttributesFavorLocal             value('00000000'xu),
       msidbFeatureAttributesFavorSource            value('00000001'xu),
       msidbFeatureAttributesFollowParent           value('00000002'xu),
       msidbFeatureAttributesFavorAdvertise         value('00000004'xu),
       msidbFeatureAttributesDisallowAdvertise      value('00000008'xu),
       msidbFeatureAttributesUIDisallowAbsent       value('00000010'xu),
       msidbFeatureAttributesNoUnsupportedAdvertise value('00000020'xu)
     ) type msidbFeatureAttributes;

 /* File.Attributes */
 define alias msidbFileAttributes                fixed bin(32) unsigned;
 dcl ( msidbFileAttributesReadOnly                  value('00000001'xu),
       msidbFileAttributesHidden                    value('00000002'xu),
       msidbFileAttributesSystem                    value('00000004'xu),
       msidbFileAttributesReserved1                 value('00000040'xu),
          /* Internal use only - must be 0 */
       msidbFileAttributesReserved2                 value('00000080'xu),
          /* Internal use only - must be 0 */
       msidbFileAttributesReserved3                 value('00000100'xu),
          /* Internal use only - must be 0 */
       msidbFileAttributesVital                     value('00000200'xu),
       msidbFileAttributesChecksum                  value('00000400'xu),
       msidbFileAttributesPatchAdded                value('00001000'xu),
          /* Internal use only - set by patches */
       msidbFileAttributesNoncompressed             value('00002000'xu),
       msidbFileAttributesCompressed                value('00004000'xu),
       msidbFileAttributesReserved4                 value('00008000'xu)
          /* Internal use only - must be 0 */
     ) type msidbFileAttributes;

 /* IniFile.Action */
 /* RemoveIniFile.Action */
 define alias msidbIniFileAction                 fixed bin(32) unsigned;
 dcl ( msidbIniFileActionAddLine                    value('00000000'xu),
       msidbIniFileActionCreateLine                 value('00000001'xu),
       msidbIniFileActionRemoveLine                 value('00000002'xu),
       msidbIniFileActionAddTag                     value('00000003'xu),
       msidbIniFileActionRemoveTag                  value('00000004'xu)
     ) type msidbIniFileAction;

 /* MoveFile.Options */
 define alias msidbMoveFileOptions               fixed bin(32) unsigned;
 dcl ( msidbMoveFileOptionsMove                     value('00000001'xu)
     ) type msidbMoveFileOptions;

 /* ODBCDataSource.Registration */
 define alias msidbODBCDataSourceRegistration    fixed bin(32) unsigned;
 dcl ( msidbODBCDataSourceRegistrationPerMachine    value('00000000'xu),
       msidbODBCDataSourceRegistrationPerUser       value('00000001'xu)
     ) type msidbODBCDataSourceRegistration;

 /* Patch.Attributes */
 define alias msidbPatchAttributes               fixed bin(32) unsigned;
 dcl ( msidbPatchAttributesNonVital                 value('00000001'xu)
     ) type msidbPatchAttributes;

 /* Registry.Root */
 /* RegLocator.Root */
 /* RemoveRegistry.Root */
 define ordinal msidbRegistryRoot
   ( msidbRegistryRootClassesRoot   value(0),
     msidbRegistryRootCurrentUser   value(1),
     msidbRegistryRootLocalMachine  value(2),
     msidbRegistryRootUsers         value(3)
   ) prec (31);

 /* RemoveFile.InstallMode */
 define alias msidbRemoveFileInstallMode         fixed bin(32) unsigned;
  dcl ( msidbRemoveFileInstallModeOnInstall         value('00000001'xu),
        msidbRemoveFileInstallModeOnRemove          value('00000002'xu),
        msidbRemoveFileInstallModeOnBoth            value('00000003'xu)
      ) type msidbRemoveFileInstallMode;

 /* ServiceControl.Event */
 define alias msidbServiceControlEvent           fixed bin(32) unsigned;
 dcl ( msidbServiceControlEventStart                value('00000001'xu),
       msidbServiceControlEventStop                 value('00000002'xu),
       msidbServiceControlEventDelete               value('00000008'xu),
       msidbServiceControlEventUninstallStart       value('00000010'xu),
       msidbServiceControlEventUninstallStop        value('00000020'xu),
       msidbServiceControlEventUninstallDelete      value('00000080'xu)
     ) type msidbServiceControlEvent;

 /* ServiceInstall.ErrorControl */
 define alias msidbServiceInstallErrorControl    fixed bin(32) unsigned;
 dcl ( msidbServiceInstallErrorControlVital         value('00008000'xu)
     ) type msidbServiceInstallErrorControl;

 /* TextStyle.StyleBits */
 define alias msidbTextStyleStyleBits            fixed bin(32) unsigned;
 dcl ( msidbTextStyleStyleBitsBold                  value('00000001'xu),
       msidbTextStyleStyleBitsItalic                value('00000002'xu),
       msidbTextStyleStyleBitsUnderline             value('00000004'xu),
       msidbTextStyleStyleBitsStrike                value('00000008'xu)
     ) type msidbTextStyleStyleBits;


 /* _________________________________________________________________ */
 /*                                                                   */
 /* SUMMARY INFORMATION PROPERTY DEFINITIONS                          */
 /* _________________________________________________________________ */

 define alias msidbSumInfoSourceType             fixed bin(32) unsigned;
 dcl ( msidbSumInfoSourceTypeSFN                    value('00000001'xu),
          /* source uses short filenames */
       msidbSumInfoSourceTypeCompressed             value('00000002'xu),
          /* source is compressed */
       msidbSumInfoSourceTypeAdminImage             value('00000004'xu)
          /* source is an admin image */
     ) type msidbSumInfoSourceType;

 %end; /* __MSIDEFS */
