 /* ++ BUILD Version: 0000                                            */
 /*   Increment this if a change has global effects                   */
 /*                                                                   */
 /* Copyright (c) 1990-1999  Microsoft Corporation                    */
 /*                                                                   */
 /* Module Name:                                                      */
 /*                                                                   */
 /*     ntsecapi.cpy                                                  */
 /*                                                                   */
 /* Abstract:                                                         */
 /*                                                                   */
 /*     This module defines the Local Security Authority APIs.        */
 /*                                                                   */
 /* Revision History:                                                 */
 /*                                                                   */
 /* --                                                                */

 %dcl #NTSECAPI_ char ext;
 %if #NTSECAPI_ = '' %then %do;
 %#NTSECAPI_ = 'Y';

 %dcl #NTSUBAUTH_ char ext;
 %if #NTSUBAUTH_ = '' %then %do;
 %include subauth;
 %end;

 %dcl #NTDEF_ char ext;
 %if #NTDEF_ = '' %then %do;
 /* already defined in subauth, included above */
 /* define alias PNTSTATUS pointer  -> LONG */
 %end;

 %dcl #NTLSA_IFS_ char ext;
 %if #NTLSA_IFS_ = '' %then %do;
 /* begin_ntifs */

 /*                                                                   */
 /* Security operation mode of the system is held in a control        */
 /* longword.                                                         */
 /*                                                                   */

 define alias  LSA_OPERATIONAL_MODE type ULONG;
 define alias PLSA_OPERATIONAL_MODE pointer; /* LSA_OPERATIONAL_MODE */

 /* end_ntifs */
 %end; /* _NTLSA_IFS_ */

 /*                                                                   */
 /* The flags in the security operational mode are defined as:        */
 /*                                                                   */
 /*    PasswordProtected - Some level of authentication (such as      */
 /*        a password) must be provided by users before they are      */
 /*        allowed to use the system.  Once set, this value will      */
 /*        not be cleared without re-booting the system.              */
 /*                                                                   */
 /*    IndividualAccounts - Each user must identify an account to     */
 /*        logon to.  This flag is only meaningful if the             */
 /*        PasswordProtected flag is also set.  If this flag is       */
 /*        not set and the PasswordProtected flag is set, then all    */
 /*        users may logon to the same account.  Once set, this value */
 /*        will not be cleared without re-booting the system.         */
 /*                                                                   */
 /*    MandatoryAccess - Indicates the system is running in a         */
 /*        mandatory access control mode (e.g., B-level as defined    */
 /*        by the U.S.A's Department of Defense's "Orange Book").     */
 /*        This is not utilized in the current release of NT.         */
 /*        This flag is only meaningful if both the PasswordProtected */
 /*        and IndividualAccounts flags are set. Once set, this value */
 /*        will not be cleared without re-booting the system.         */
 /*                                                                   */
 /*    LogFull - Indicates the system has been brought up in a mode   */
 /*        in which if must perform security auditing, but its audit  */
 /*        log is full.  This may (should) restrict the operations    */
 /*        that can occur until the audit log is made not-full again. */
 /*        THIS VALUE MAY BE CLEARED WHILE THE SYSTEM IS RUNNING      */
 /*        (I.E., WITHOUT REBOOTING).                                 */
 /*                                                                   */
 /* If the PasswordProtected flag is not set, then the system is      */
 /* running without security, and user interface should be adjusted   */
 /* appropriately.                                                    */
 /*                                                                   */

 dcl LSA_MODE_PASSWORD_PROTECTED  type DWORD value('00000001'xu);
 dcl LSA_MODE_INDIVIDUAL_ACCOUNTS type DWORD value('00000002'xu);
 dcl LSA_MODE_MANDATORY_ACCESS    type DWORD value('00000004'xu);
 dcl LSA_MODE_LOG_FULL            type DWORD value('00000008'xu);

 %dcl #NTLSA_IFS_ char ext;
 %if #NTLSA_IFS_ = '' %then %do;
 /* begin_ntifs */
 /*                                                                   */
 /* Used by a logon process to indicate what type of logon is being   */
 /* requested.                                                        */
 /*                                                                   */

 define ordinal
   SECURITY_LOGON_TYPE
     ( Interactive value(2), /* Interactively logged on */
                         /* (locally or remotely) */
       Network,          /* Accessing system via network */
       Batch,            /* Started via a batch queue */
       Service,          /* Service started by service controller */
       Proxy,            /* Proxy logon */
       Unlock,           /* Unlock workstation */
       NetworkCleartext, /* Network logon with cleartext credentials */
       NewCredentials);  /* Clone caller, new default credentials */
 dcl PSECURITY_LOGON_TYPE pointer; /* to SECURITY_LOGON_TYPE */

 /* end_ntifs */
 %end; /* _NTLSA_IFS_ */


 /*                                                                   */
 /* Audit Event Categories                                            */
 /*                                                                   */
 /* The following are the built-in types or Categories of audit event.*/
 /* WARNING!  This structure is subject to expansion. The user should */
 /* not compute the number of elements of this type directly, but     */
 /* instead should obtain the count of elements by calling            */
 /* LsaQueryInformationPolicy() for the PolicyAuditEventsInformation  */
 /* class and extracting the count from the MaximumAuditEventCount    */
 /* field of the returned structure.                                  */
 /*                                                                   */

 define ordinal
   POLICY_AUDIT_EVENT_TYPE
     ( AuditCategorySystem,
       AuditCategoryLogon,
       AuditCategoryObjectAccess,
       AuditCategoryPrivilegeUse,
       AuditCategoryDetailedTracking,
       AuditCategoryPolicyChange,
       AuditCategoryAccountManagement,
       AuditCategoryDirectoryServiceAccess,
       AuditCategoryAccountLogon);
 dcl PPOLICY_AUDIT_EVENT_TYPE pointer; /* to POLICY_AUDIT_EVENT_TYPE */


 /*                                                                   */
 /* The following defines describe the auditing options for each      */
 /* event type                                                        */
 /*                                                                   */

 /* Leave options specified for this event unchanged */

 dcl POLICY_AUDIT_EVENT_UNCHANGED type DWORD value('00000000'xu);

 /* Audit successful occurrences of events of this type */

 dcl POLICY_AUDIT_EVENT_SUCCESS   type DWORD value('00000001'xu);

 /* Audit failed attempts to cause an event of this type to occur */

 dcl POLICY_AUDIT_EVENT_FAILURE   type DWORD value('00000002'xu);

 dcl POLICY_AUDIT_EVENT_NONE      type DWORD value('00000004'xu);

 /* Mask of valid event auditing options */

 dcl POLICY_AUDIT_EVENT_MASK      type DWORD
     value (ior (POLICY_AUDIT_EVENT_SUCCESS,
                 POLICY_AUDIT_EVENT_FAILURE,
                 POLICY_AUDIT_EVENT_UNCHANGED,
                 POLICY_AUDIT_EVENT_NONE));



 %dcl #NTDEF_ char ext;
 %if #NTDEF_ ^= '' %then %do;
 /* begin_ntifs */
 define alias PLSA_UNICODE_STRING    handle UNICODE_STRING;
 define alias PLSA_STRING            handle STRING;
 define alias PLSA_OBJECT_ATTRIBUTES handle OBJECT_ATTRIBUTES;
 /* end_ntifs */
 %end; %else %do;

 define structure
   1 LSA_UNICODE_STRING,
     2 Length        type USHORT,
     2 MaximumLength type USHORT,
     2 Buffer        type PWSTR;
 define alias PLSA_UNICODE_STRING handle LSA_UNICODE_STRING;

 define structure
   1 LSA_STRING,
     2 Length        type USHORT,
     2 MaximumLength type USHORT,
     2 Buffer        type PCHAR;
 define alias PLSA_STRING handle LSA_STRING;

 define structure
   1 LSA_OBJECT_ATTRIBUTES,
     2 Length                   type ULONG,
     2 RootDirectory            type HANDLE,
     2 ObjectName               type PLSA_UNICODE_STRING,
     2 Attributes               type ULONG,
     2 SecurityDescriptor       type PVOID,
           /* Points to type SECURITY_DESCRIPTOR */
     2 SecurityQualityOfService type PVOID;
           /* Points to type SECURITY_QUALITY_OF_SERVICE */
 define alias PLSA_OBJECT_ATTRIBUTES handle LSA_OBJECT_ATTRIBUTES;


 %end; /* _NTDEF_ */

 /*                                                                   */
 /* Macro for determining whether an API succeeded.                   */
 /*                                                                   */

 LSA_SUCCESS: proc (Error) returns (bit (1) aligned byvalue)
              options(inline) internal;
   dcl Error type LONG byvalue;
   return (Error >= 0);
 end LSA_SUCCESS;


 %dcl #NTLSA_IFS_ char ext;
 %if #NTLSA_IFS_ = '' %then %do;
 /* begin_ntifs */

 dcl LsaRegisterLogonProcess
     entry (type LSA_STRING byaddr nonasgn,
            type HANDLE byaddr,
            type LSA_OPERATIONAL_MODE byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaRegisterLogonProcess');

 /* end_ntifs */
 /* begin_ntsrv */

 dcl LsaLogonUser
     entry (type HANDLE,
            type LSA_STRING byaddr nonasgn,
            type SECURITY_LOGON_TYPE,
            type ULONG,
            type PVOID,
            type ULONG,
            type TOKEN_GROUPS byaddr nonasgn optional,
            type TOKEN_SOURCE byaddr nonasgn,
            type PVOID byaddr,
            type ULONG byaddr,
            type LUID byaddr,
            type HANDLE byaddr,
            type QUOTA_LIMITS byaddr,
            type NTSTATUS byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaLogonUser');

 /* end_ntsrv */

 dcl LsaLookupAuthenticationPackage
     entry (type HANDLE,
            type LSA_STRING byaddr nonasgn,
            type ULONG byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaLookupAuthenticationPackage');

 /* begin_ntifs */

 dcl LsaFreeReturnBuffer
     entry (type PVOID)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaFreeReturnBuffer');

 /* end_ntifs */

 dcl LsaCallAuthenticationPackage
     entry (type HANDLE,
            type ULONG,
            type PVOID,
            type ULONG,
            type PVOID byaddr,
            type ULONG byaddr,
            type NTSTATUS byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaCallAuthenticationPackage');


 dcl LsaDeregisterLogonProcess
     entry (type HANDLE)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaDeregisterLogonProcess');

 dcl LsaConnectUntrusted
     entry (type PHANDLE)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaConnectUntrusted');

 %end; /* _NTLSA_IFS_ */


 /*********************************************************************/
 /*                                                                   */
 /* Local Security Policy Administration API datatypes and defines    */
 /*                                                                   */
 /*********************************************************************/

 /*                                                                   */
 /* Access types for the Policy object                                */
 /*                                                                   */

 dcl POLICY_VIEW_LOCAL_INFORMATION   type ULONG value('00000001'xu);
 dcl POLICY_VIEW_AUDIT_INFORMATION   type ULONG value('00000002'xu);
 dcl POLICY_GET_PRIVATE_INFORMATION  type ULONG value('00000004'xu);
 dcl POLICY_TRUST_ADMIN              type ULONG value('00000008'xu);
 dcl POLICY_CREATE_ACCOUNT           type ULONG value('00000010'xu);
 dcl POLICY_CREATE_SECRET            type ULONG value('00000020'xu);
 dcl POLICY_CREATE_PRIVILEGE         type ULONG value('00000040'xu);
 dcl POLICY_SET_DEFAULT_QUOTA_LIMITS type ULONG value('00000080'xu);
 dcl POLICY_SET_AUDIT_REQUIREMENTS   type ULONG value('00000100'xu);
 dcl POLICY_AUDIT_LOG_ADMIN          type ULONG value('00000200'xu);
 dcl POLICY_SERVER_ADMIN             type ULONG value('00000400'xu);
 dcl POLICY_LOOKUP_NAMES             type ULONG value('00000800'xu);
 dcl POLICY_NOTIFICATION             type ULONG value('00001000'xu);

 dcl POLICY_ALL_ACCESS
     type ULONG value (ior (STANDARD_RIGHTS_REQUIRED,
                            POLICY_VIEW_LOCAL_INFORMATION,
                            POLICY_VIEW_AUDIT_INFORMATION,
                            POLICY_GET_PRIVATE_INFORMATION,
                            POLICY_TRUST_ADMIN,
                            POLICY_CREATE_ACCOUNT,
                            POLICY_CREATE_SECRET,
                            POLICY_CREATE_PRIVILEGE,
                            POLICY_SET_DEFAULT_QUOTA_LIMITS,
                            POLICY_SET_AUDIT_REQUIREMENTS,
                            POLICY_AUDIT_LOG_ADMIN,
                            POLICY_SERVER_ADMIN,
                            POLICY_LOOKUP_NAMES));

 dcl POLICY_READ
     type ULONG value (ior (STANDARD_RIGHTS_READ,
                            POLICY_VIEW_AUDIT_INFORMATION,
                            POLICY_GET_PRIVATE_INFORMATION));

 dcl POLICY_WRITE
     type ULONG value (ior (STANDARD_RIGHTS_WRITE,
                            POLICY_TRUST_ADMIN,
                            POLICY_CREATE_ACCOUNT,
                            POLICY_CREATE_SECRET,
                            POLICY_CREATE_PRIVILEGE,
                            POLICY_SET_DEFAULT_QUOTA_LIMITS,
                            POLICY_SET_AUDIT_REQUIREMENTS,
                            POLICY_AUDIT_LOG_ADMIN,
                            POLICY_SERVER_ADMIN));

 dcl POLICY_EXECUTE
     type ULONG value (ior (STANDARD_RIGHTS_EXECUTE,
                            POLICY_VIEW_LOCAL_INFORMATION,
                            POLICY_LOOKUP_NAMES));

 /*                                                                   */
 /* Policy object specific data types.                                */
 /*                                                                   */

 /*                                                                   */
 /* The following data type is used to identify a domain              */
 /*                                                                   */

 define structure
   1 LSA_TRUST_INFORMATION,
     2 Name type LSA_UNICODE_STRING,
     2 Sid  type PSID;
 define alias PLSA_TRUST_INFORMATION handle LSA_TRUST_INFORMATION;

 /* where members have the following usage:                           */
 /*                                                                   */
 /*     Name - The name of the domain.                                */
 /*                                                                   */
 /*     Sid - A pointer to the Sid of the Domain                      */
 /*                                                                   */

 /*                                                                   */
 /* The following data type is used in name and SID lookup services   */
 /* to describe the domains referenced in the lookup operation.       */
 /*                                                                   */

 define structure
   1 LSA_REFERENCED_DOMAIN_LIST,
     2 Entries type ULONG,
     2 Domains type PLSA_TRUST_INFORMATION;
 define alias PLSA_REFERENCED_DOMAIN_LIST
        handle LSA_REFERENCED_DOMAIN_LIST;

 /* where members have the following usage:                           */
 /*                                                                   */
 /*     Entries - Is a count of the number of domains described in    */
 /*         the Domains array.                                        */
 /*                                                                   */
 /*     Domains - Is a pointer to an array of Entries
 /*         LSA_TRUST_INFORMATION data structures.                    */
 /*                                                                   */


 /*                                                                   */
 /* The following data type is used in name to SID lookup services    */
 /* to describe the domains referenced in the lookup operation.       */
 /*                                                                   */

 define structure
   1 LSA_TRANSLATED_SID,
     2 Use         type SID_NAME_USE,
     2 RelativeId  type ULONG,
     2 DomainIndex type LONG;
 define alias PLSA_TRANSLATED_SID handle LSA_TRANSLATED_SID;

 /* where members have the following usage:                           */
 /*                                                                   */
 /*     Use - identifies the use of the SID.  If this value is        */
 /*         SidUnknown or SidInvalid, then the remainder of the       */
 /*         record is not set and should be ignored.                  */
 /*                                                                   */
 /*     RelativeId - Contains the relative ID of the translated SID.  */
 /*         The remainder of the SID (the prefix) is obtained using   */
 /*         the DomainIndex field.                                    */
 /*                                                                   */
 /*     DomainIndex - Is the index of an entry in a related           */
 /*         LSA_REFERENCED_DOMAIN_LIST data structure describing the  */
 /*         domain in which the account was found.                    */
 /*                                                                   */
 /*         If there is no corresponding reference domain for an      */
 /*         entry, then this field will contain a negative value.     */
 /*                                                                   */


 /*                                                                   */
 /* The following data type is used in SID to name lookup services to */
 /* describe the domains referenced in the lookup operation.          */
 /*                                                                   */

 define structure
   1 LSA_TRANSLATED_NAME,
     2 Use         type SID_NAME_USE,
     2 Name        type LSA_UNICODE_STRING,
     2 DomainIndex type LONG;
 define alias PLSA_TRANSLATED_NAME handle LSA_TRANSLATED_NAME;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*     Use - Identifies the use of the name.  If this value is
 /*         SidUnknown or SidInvalid, then the remainder of the       */
 /*         record is not set and should be ignored.  If this value   */
 /*         is SidWellKnownGroup then the Name field is invalid, but  */
 /*         the DomainIndex field is not.                             */
 /*                                                                   */
 /*     Name - Contains the isolated name of the translated SID.      */
 /*                                                                   */
 /*     DomainIndex - Is the index of an entry in a related           */
 /*         LSA_REFERENCED_DOMAIN_LIST data structure describing the  */
 /*         domain in which the account was found.                    */
 /*                                                                   */
 /*         If there is no corresponding reference domain for an      */
 /*         entry, then This field will contain a negative value.     */
 /*                                                                   */


 /*                                                                   */
 /* The following data type is used to represent the role of the LSA  */
 /* server (primary or backup).                                       */
 /*                                                                   */

 define ordinal
   POLICY_LSA_SERVER_ROLE
     ( PolicyServerRoleBackup  value(2),
       PolicyServerRolePrimary);
 dcl PPOLICY_LSA_SERVER_ROLE pointer; /* to POLICY_LSA_SERVER_ROLE */


 /*                                                                   */
 /* The following data type is used to represent the state of the LSA */
 /* server (enabled or disabled).  Some operations may only be        */
 /* performed on an enabled LSA server.                               */
 /*                                                                   */

 define ordinal
   POLICY_SERVER_ENABLE_STATE
     ( PolicyServerEnabled  value(2),
       PolicyServerDisabled);
 dcl PPOLICY_SERVER_ENABLE_STATE pointer;
                                     /* to POLICY_SERVER_ENABLE_STATE */


 /*                                                                   */
 /* The following data type is used to specify the auditing options   */
 /* for an Audit Event Type.                                          */
 /*                                                                   */

 define alias PPOLICY_AUDIT_EVENT_OPTIONS pointer; /* to ULONG */

 /* where the following flags can be set:                             */
 /*                                                                   */
 /*     POLICY_AUDIT_EVENT_UNCHANGED - Leave existing auditing        */
 /*         options unchanged for events of this type.  This flag is  */
 /*         only used for set operations.  If this flag is set, then  */
 /*         all other flags are ignored.                              */
 /*                                                                   */
 /*     POLICY_AUDIT_EVENT_NONE - Cancel all auditing options for     */
 /*         events of this type.  If this flag is set, the            */
 /*         success/failure flags are ignored.                        */
 /*                                                                   */
 /*     POLICY_AUDIT_EVENT_SUCCESS - When auditing is enabled, audit  */
 /*         all successful occurrences of events of the given type.   */
 /*                                                                   */
 /*     POLICY_AUDIT_EVENT_FAILURE - When auditing is enabled, audit  */
 /*         all unsuccessful occurrences of events of the given type. */
 /*                                                                   */


 /*                                                                   */
 /* The following data type defines the classes of Policy Information */
 /* that may be queried/set.                                          */
 /*                                                                   */

 define ordinal
   POLICY_INFORMATION_CLASS
     ( PolicyAuditLogInformation value(1),
       PolicyAuditEventsInformation,
       PolicyPrimaryDomainInformation,
       PolicyPdAccountInformation,
       PolicyAccountDomainInformation,
       PolicyLsaServerRoleInformation,
       PolicyReplicaSourceInformation,
       PolicyDefaultQuotaInformation,
       PolicyModificationInformation,
       PolicyAuditFullSetInformation,
       PolicyAuditFullQueryInformation,
       PolicyDnsDomainInformation);
 dcl PPOLICY_INFORMATION_CLASS pointer; /* to POLICY_INFORMATION_CLASS*/


 /*                                                                   */
 /* The following data type corresponds to the                        */
 /* PolicyAuditLogInformation information class.                      */
 /* It is used to represent information relating to the Audit Log.    */
 /*                                                                   */
 /* This structure may be used in both query and set operations.      */
 /* However,when used in set operations, some fields are ignored.     */
 /*                                                                   */

 define structure
   1 POLICY_AUDIT_LOG_INFO,
     2 AuditLogPercentFull            type ULONG,
     2 MaximumLogSize                 type ULONG,
     2 AuditRetentionPeriod           type LARGE_INTEGER,
     2 AuditLogFullShutdownInProgress type BOOLEAN,
     2 TimeToShutdown                 type LARGE_INTEGER,
     2 NextAuditRecordId              type ULONG;
 define alias PPOLICY_AUDIT_LOG_INFO handle POLICY_AUDIT_LOG_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*     AuditLogPercentFull - Indicates the percentage of the Audit   */
 /*         Log currently being used.                                 */
 /*                                                                   */
 /*     MaximumLogSize - Specifies the maximum size of the Audit Log  */
 /*         in kilobytes.                                             */
 /*                                                                   */
 /*     AuditRetentionPeriod - Indicates the length of time that      */
 /*         Audit Records are to be retained.  Audit Records are      */
 /*         discardable if their timestamp predates the current time  */
 /*         minus the retention period.                               */
 /*                                                                   */
 /*     AuditLogFullShutdownInProgress - Indicates whether or not a   */
 /*         system shutdown is being initiated due to the security    */
 /*         Audit Log becoming full.  This condition will only occur  */
 /*         if the system is configured TO shutdown when the log      */
 /*         becomes full.                                             */
 /*                                                                   */
 /*         TRUE indicates that a shutdown is in progress             */
 /*         FALSE indicates that a shutdown is not in progress.       */
 /*                                                                   */
 /*         Once a shutdown has been initiated, this flag will be set */
 /*         to TRUE.  If an administrator is able to currect the      */
 /*         situation before the shutdown becomes irreversible, then  */
 /*         this flag will be reset to false.                         */
 /*                                                                   */
 /*         This field is ignored for set operations.                 */
 /*                                                                   */
 /*     TimeToShutdown - If the AuditLogFullShutdownInProgress flag   */
 /*         is set, then this field contains the time left before the */
 /*         shutdown becomes irreversible.                            */
 /*                                                                   */
 /*         This field is ignored for set operations.                 */
 /*                                                                   */


 /*                                                                   */
 /* The following data type corresponds to the                        */
 /* PolicyAuditEventsInformation information class.  It is used to    */
 /* represent information relating to the audit requirements.         */
 /*                                                                   */

 define structure
   1 POLICY_AUDIT_EVENTS_INFO,
     2 AuditingMode           type BOOLEAN,
     2 EventAuditingOptions   type PPOLICY_AUDIT_EVENT_OPTIONS,
     2 MaximumAuditEventCount type ULONG;
 define alias PPOLICY_AUDIT_EVENTS_INFO handle POLICY_AUDIT_EVENTS_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*     AuditingMode - A Boolean variable specifying the Auditing     */
 /*         Mode value. This value is interpreted as follows:         */
 /*                                                                   */
 /*         TRUE - Auditing is to be enabled (set operations) or is   */
 /*             enabled (query operations).  Audit Records will be    */
 /*             generated according to the Event Auditing Options in  */
 /*             effect (see the EventAuditingOptions field.           */
 /*                                                                   */
 /*         FALSE - Auditing is to be disabled (set operations) or is */
 /*             disabled (query operations).  No Audit Records will   */
 /*             be generated.  Note that for set operations the Event */
 /*             Auditing Options in effect will still be updated as   */
 /*             specified by the EventAuditingOptions field whether   */
 /*             Auditing is enabled or disabled.                      */
 /*                                                                   */
 /*    EventAuditingOptions - Pointer to an array of Auditing Options */
 /*        indexed by Audit Event Type.                               */
 /*                                                                   */
 /*    MaximumAuditEventCount - Specifies a count of the number of    */
 /*        Audit Event Types specified by the EventAuditingOptions    */
 /*        parameter. If this count is less than the number of Audit  */
 /*        Event Types supported by the system, the Auditing Options  */
 /*        for Event Types with IDs higher than                       */
 /*        (MaximumAuditEventCount + 1) are left unchanged.           */
 /*                                                                   */


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyAccountDomainInformation information class.                 */
 /*                                                                   */

 define structure
   1 POLICY_ACCOUNT_DOMAIN_INFO,
     2 DomainName type LSA_UNICODE_STRING,
     2 DomainSid  type PSID;
 define alias PPOLICY_ACCOUNT_DOMAIN_INFO
        handle POLICY_ACCOUNT_DOMAIN_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*     DomainName - Is the name of the domain                        */
 /*                                                                   */
 /*     DomainSid - Is the Sid of the domain                          */
 /*                                                                   */


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyPrimaryDomainInformation information class.                 */
 /*                                                                   */

 define structure
   1 POLICY_PRIMARY_DOMAIN_INFO,
     2 Name type LSA_UNICODE_STRING,
     2 Sid  type PSID;
 define alias PPOLICY_PRIMARY_DOMAIN_INFO
        handle POLICY_PRIMARY_DOMAIN_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*     Name - Is the name of the domain                              */
 /*                                                                   */
 /*     Sid - Is the Sid of the domain                                */
 /*                                                                   */


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyDnsDomainInformation information class                      */
 /*                                                                   */

 define structure
   1 POLICY_DNS_DOMAIN_INFO,
     2 Name          type LSA_UNICODE_STRING,
     2 DnsDomainName type LSA_UNICODE_STRING,
     2 DnsForestName type LSA_UNICODE_STRING,
     2 DomainGuid    type GUID,
     2 Sid           type PSID;
 define alias PPOLICY_DNS_DOMAIN_INFO handle POLICY_DNS_DOMAIN_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*      Name - Is the name of the Domain                             */
 /*                                                                   */
 /*      DnsDomainName - Is the DNS name of the domain                */
 /*                                                                   */
 /*      DnsForestName - Is the DNS forest name of the domain         */
 /*                                                                   */
 /*      DomainGuid - Is the GUID of the domain                       */
 /*                                                                   */
 /*      Sid - Is the Sid of the domain                               */


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyPdAccountInformation information class.                     */
 /* This structure may be used in Query operations only.              */
 /*                                                                   */

 define structure
   1 POLICY_PD_ACCOUNT_INFO,
     2 Name type LSA_UNICODE_STRING;
 define alias PPOLICY_PD_ACCOUNT_INFO handle POLICY_PD_ACCOUNT_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*     Name - Is the name of an account in the domain that should be */
 /*         used for authentication and name/ID lookup requests.      */
 /*                                                                   */


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyLsaServerRoleInformation information class.                 */
 /*                                                                   */

 define structure
   1 POLICY_LSA_SERVER_ROLE_INFO,
     2 LsaServerRole type POLICY_LSA_SERVER_ROLE;
 define alias PPOLICY_LSA_SERVER_ROLE_INFO
        handle POLICY_LSA_SERVER_ROLE_INFO;

 /* where the fields have the following usage:                        */
 /*                                                                   */
 /* TBS                                                               */
 /*                                                                   */


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyReplicaSourceInformation information class.                 */
 /*                                                                   */

 define structure
   1 POLICY_REPLICA_SOURCE_INFO,
     2 ReplicaSource type LSA_UNICODE_STRING,
     2 ReplicaAccountName type LSA_UNICODE_STRING;
 define alias PPOLICY_REPLICA_SOURCE_INFO
        handle POLICY_REPLICA_SOURCE_INFO;

 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyDefaultQuotaInformation information class.                  */
 /*                                                                   */

 define structure
   1 POLICY_DEFAULT_QUOTA_INFO,
     2 QuotaLimits type QUOTA_LIMITS;
 define alias PPOLICY_DEFAULT_QUOTA_INFO
        handle POLICY_DEFAULT_QUOTA_INFO;


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyModificationInformation information class.                  */
 /*                                                                   */

 define structure
   1 POLICY_MODIFICATION_INFO,
     2 ModifiedId           type LARGE_INTEGER,
     2 DatabaseCreationTime type LARGE_INTEGER;
 define alias PPOLICY_MODIFICATION_INFO handle POLICY_MODIFICATION_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*     ModifiedId - Is a 64-bit unsigned integer that is incremented */
 /*         each time anything in the LSA database is modified.       */
 /*         This value is only modified on Primary Domain Controllers.*/
 /*                                                                   */
 /*     DatabaseCreationTime - Is the date/time that the LSA Database */
 /*         was created.  On Backup Domain Controllers, this value is */
 /*         replicated from the Primary Domain Controller.            */
 /*                                                                   */

 /*                                                                   */
 /* The following structure type corresponds to the                   */
 /* PolicyAuditFullSetInformation Information Class.                  */
 /*                                                                   */

 define structure
   1 POLICY_AUDIT_FULL_SET_INFO,
     2 ShutDownOnFull type BOOLEAN;
 define alias PPOLICY_AUDIT_FULL_SET_INFO
        handle POLICY_AUDIT_FULL_SET_INFO;

 /*                                                                   */
 /* The following structure type corresponds to the                   */
 /* PolicyAuditFullQueryInformation Information Class.                */
 /*                                                                   */

 define structure
   1 POLICY_AUDIT_FULL_QUERY_INFO,
     2 ShutDownOnFull type BOOLEAN,
     2 LogIsFull      type BOOLEAN;
 define alias PPOLICY_AUDIT_FULL_QUERY_INFO
        handle POLICY_AUDIT_FULL_QUERY_INFO;

 /*                                                                   */
 /* The following data type defines the classes of Policy Information */
 /* that may be queried/set that has domain wide effect.              */
 /*                                                                   */

 define ordinal
   POLICY_DOMAIN_INFORMATION_CLASS
     ( PolicyDomainQualityOfServiceInformation value(1),
       PolicyDomainEfsInformation,
       PolicyDomainKerberosTicketInformation);
 define alias PPOLICY_DOMAIN_INFORMATION_CLASS pointer;
        /* to POLICY_DOMAIN_INFORMATION_CLASS */


 /*                                                                   */
 /* QualityOfService information.  Corresponds to PolicyDomainQualityOfS
 erviceInformation */
 /*                                                                   */

 dcl POLICY_QOS_SCHANNEL_REQUIRED        type ULONG value('00000001'xu);
 dcl POLICY_QOS_OUTBOUND_INTEGRITY       type ULONG value('00000002'xu);
 dcl POLICY_QOS_OUTBOUND_CONFIDENTIALITY type ULONG value('00000004'xu);
 dcl POLICY_QOS_INBOUND_INTEGRITY        type ULONG value('00000008'xu);
 dcl POLICY_QOS_INBOUND_CONFIDENTIALITY  type ULONG value('00000010'xu);
 dcl POLICY_QOS_ALLOW_LOCAL_ROOT_CERT_STORE
                                         type ULONG value('00000020'xu);
 dcl POLICY_QOS_RAS_SERVER_ALLOWED       type ULONG value('00000040'xu);
 dcl POLICY_QOS_DHCP_SERVER_ALLOWED      type ULONG value('00000080'xu);

 /*                                                                   */
 /* Bits 0x00000100 through 0xFFFFFFFF are reserved for future use.   */
 /*                                                                   */

 define structure
   1 POLICY_DOMAIN_QUALITY_OF_SERVICE_INFO,
     2 QualityOfService type ULONG;
 define alias PPOLICY_DOMAIN_QUALITY_OF_SERVICE_INFO
        handle POLICY_DOMAIN_QUALITY_OF_SERVICE_INFO;
 /*                                                                   */
 /* where the members have the following usage:                       */
 /*                                                                   */
 /*  QualityOfService - Determines what specific QOS actions a        */
 /*  machine should take                                              */
 /*                                                                   */


 /*                                                                   */
 /* The following structure corresponds to the PolicyEfsInformation   */
 /* information class                                                 */
 /*                                                                   */

 define structure
   1 POLICY_DOMAIN_EFS_INFO,
     2 InfoLength type ULONG,
     2 EfsBlob    type PUCHAR;
 define alias PPOLICY_DOMAIN_EFS_INFO handle POLICY_DOMAIN_EFS_INFO;

 /* where the members have the following usage:                       */
 /*                                                                   */
 /*      InfoLength - Length of the EFS Information blob              */
 /*                                                                   */
 /*      EfsBlob - Efs blob data                                      */
 /*                                                                   */


 /*                                                                   */
 /* The following structure corresponds to the                        */
 /* PolicyDomainKerberosTicketInformation information class           */

 dcl POLICY_KERBEROS_VALIDATE_CLIENT type ULONG value('00000080'xu);

 define structure
   1 POLICY_DOMAIN_KERBEROS_TICKET_INFO,
     2 AuthenticationOptions type ULONG,
     2 MaxServiceTicketAge   type LARGE_INTEGER,
     2 MaxTicketAge          type LARGE_INTEGER,
     2 MaxRenewAge           type LARGE_INTEGER,
     2 MaxClockSkew          type LARGE_INTEGER,
     2 Reserved              type LARGE_INTEGER;
 define alias PPOLICY_DOMAIN_KERBEROS_TICKET_INFO
        handle POLICY_DOMAIN_KERBEROS_TICKET_INFO;

 /*                                                                   */
 /* where the members have the following usage                        */
 /*                                                                   */
 /*      AuthenticationOptions -- allowed ticket options              */
 /*                     (POLICY_KERBEROS_* flags )                    */
 /*                                                                   */
 /*      MaxServiceTicketAge   -- Maximum lifetime for a service      */
 /*                     ticket                                        */
 /*                                                                   */
 /*      MaxTicketAge -- Maximum lifetime for the initial ticket      */
 /*                                                                   */
 /*      MaxRenewAge -- Maximum cumulative age a renewable ticket can */
 /*                     be with requiring authentication              */
 /*                                                                   */
 /*      MaxClockSkew -- Maximum tolerance for synchronization of     */
 /*                     computer clocks                               */
 /*                                                                   */
 /*      Reserved   --  Reserved                                      */


 /*                                                                   */
 /* The following data type defines the classes of Policy Information */
 /* / Policy Domain Information                                       */
 /* that may be used to request notification                          */
 /*                                                                   */

 define ordinal
   POLICY_NOTIFICATION_INFORMATION_CLASS
     ( PolicyNotifyAuditEventsInformation value(1),
       PolicyNotifyAccountDomainInformation,
       PolicyNotifyServerRoleInformation,
       PolicyNotifyDnsDomainInformation,
       PolicyNotifyDomainEfsInformation,
       PolicyNotifyDomainKerberosTicketInformation,
       PolicyNotifyMachineAccountPasswordInformation);
 define alias PPOLICY_NOTIFICATION_INFORMATION_CLASS pointer;
        /* to POLICY_NOTIFICATION_INFORMATION_CLASS */


 /*                                                                   */
 /* LSA RPC Context Handle (Opaque form).  Note that a Context Handle */
 /* is always a pointer type unlike regular handles.                  */
 /*                                                                   */

 define alias LSA_HANDLE  type PVOID;
 define alias PLSA_HANDLE pointer; /* to LSA_HANDLE */


 /*                                                                   */
 /* Trusted Domain Object specific data types                         */
 /*                                                                   */

 /*                                                                   */
 /* This data type defines the following information classes that may */
 /* be queried or set.                                                */
 /*                                                                   */

 define ordinal
   TRUSTED_INFORMATION_CLASS
     ( TrustedDomainNameInformation value(1),
       TrustedControllersInformation,
       TrustedPosixOffsetInformation,
       TrustedPasswordInformation,
       TrustedDomainInformationBasic,
       TrustedDomainInformationEx,
       TrustedDomainAuthInformation,
       TrustedDomainFullInformation);
 dcl PTRUSTED_INFORMATION_CLASS pointer;
                                     /* to #TRUSTED_INFORMATION_CLASS */

 /*                                                                   */
 /* The following data type corresponds to the                        */
 /* TrustedDomainNameInformation information class.                   */
 /*                                                                   */

 define structure
   1 TRUSTED_DOMAIN_NAME_INFO,
     2 Name type LSA_UNICODE_STRING;
 define alias PTRUSTED_DOMAIN_NAME_INFO handle TRUSTED_DOMAIN_NAME_INFO;

 /* where members have the following meaning:                         */
 /*                                                                   */
 /* Name - The name of the Trusted Domain.                            */
 /*                                                                   */

 /*                                                                   */
 /* The following data type corresponds to the                        */
 /* TrustedControllersInformation information class.                  */
 /*                                                                   */

 define structure
   1 TRUSTED_CONTROLLERS_INFO,
     2 Entries type ULONG,
     2 Names   type PLSA_UNICODE_STRING;
 define alias PTRUSTED_CONTROLLERS_INFO handle TRUSTED_CONTROLLERS_INFO;

 /* where members have the following meaning:                         */
 /*                                                                   */
 /* Entries - Indicate how mamy entries there are in the Names array. */
 /*                                                                   */
 /* Names - Pointer to an array of LSA_UNICODE_STRING structures      */
 /*     containing the names of domain controllers of the domain.     */
 /*     This information may not be accurate and should be used only  */
 /*     as a hint.  The order of this list is considered significant  */
 /*     and will be maintained.                                       */
 /*                                                                   */
 /*     By convention, the first name in this list is assumed to be   */
 /*     the Primary Domain Controller of the domain.                  */
 /*     If the Primary Domain Controller is not known, the first name */
 /*     should be set to the NULL string.                             */
 /*                                                                   */


 /*                                                                   */
 /* The following data type corresponds to the                        */
 /* TrustedPosixOffsetInformation information class.                  */
 /*                                                                   */

 define structure
   1 TRUSTED_POSIX_OFFSET_INFO,
     2 Offset type ULONG;
 define alias PTRUSTED_POSIX_OFFSET_INFO
        handle TRUSTED_POSIX_OFFSET_INFO;

 /* where members have the following meaning:                         */
 /*                                                                   */
 /* Offset - Is an offset to use for the generation of Posix user and */
 /*     group IDs from SIDs.  The Posix ID corresponding to any       */
 /*     particular SID is generated by adding the RID of that SID to  */
 /*     the Offset of the SID's corresponding TrustedDomain object.   */
 /*                                                                   */

 /*                                                                   */
 /* The following data type corresponds to the                        */
 /* TrustedPasswordInformation information class.                     */
 /*                                                                   */

 define structure
   1 TRUSTED_PASSWORD_INFO,
     2 Password    type LSA_UNICODE_STRING,
     2 OldPassword type LSA_UNICODE_STRING;
 define alias PTRUSTED_PASSWORD_INFO handle TRUSTED_PASSWORD_INFO;

 define alias TRUSTED_DOMAIN_INFORMATION_BASIC
         type LSA_TRUST_INFORMATION;

 define alias PTRUSTED_DOMAIN_INFORMATION_BASIC
         type PLSA_TRUST_INFORMATION;

 /*                                                                   */
 /* Direction of the trust                                            */
 /*                                                                   */
 dcl TRUST_DIRECTION_DISABLED            type ULONG value('00000000'xu);
 dcl TRUST_DIRECTION_INBOUND             type ULONG value('00000001'xu);
 dcl TRUST_DIRECTION_OUTBOUND            type ULONG value('00000002'xu);
 dcl TRUST_DIRECTION_BIDIRECTIONAL type ULONG
           value (ior (TRUST_DIRECTION_INBOUND,
                       TRUST_DIRECTION_OUTBOUND));

 dcl TRUST_TYPE_DOWNLEVEL                type ULONG value('00000001'xu);
     /* NT4 and before */
 dcl TRUST_TYPE_UPLEVEL                  type ULONG value('00000002'xu);
     /* NT5 */
 dcl TRUST_TYPE_MIT                      type ULONG value('00000003'xu);
     /* Trust with a MIT Kerberos realm */
 dcl TRUST_TYPE_DCE                      type ULONG value('00000004'xu);
     /* Trust with a DCE realm */
 /* Levels 0x5 - 0x000FFFFF reserved for future use */
 /* Provider specific trust levels are from 0x00100000 to 0xFFF00000 */

 dcl TRUST_ATTRIBUTE_NON_TRANSITIVE      type ULONG value('00000001'xu);
     /* Disallow transitivity */
 dcl TRUST_ATTRIBUTE_UPLEVEL_ONLY        type ULONG value('00000002'xu);
     /* Trust link only valid for uplevel client */
 dcl TRUST_ATTRIBUTE_TREE_PARENT         type ULONG value('00400000'xu);
     /* Denotes that we are setting the trust */
     /* to our parent in the org tree...      */
 dcl TRUST_ATTRIBUTE_TREE_ROOT           type ULONG value('00800000'xu);
     /* Denotes that we are setting the trust */
     /* to another tree root in a forest...   */
 /* Trust attributes 0x00000004 through 0x004FFFFF */
 /* reserved for future use                        */
 /* Trust attributes 0x00F00000 through 0x00400000 */
 /* are reserved for internal use                  */
 /* Trust attributes 0x01000000 through 0xFF000000 */
 /* are reserved for user defined values           */
 dcl TRUST_ATTRIBUTES_VALID              type ULONG value('FF02FFFF'xu);
 dcl TRUST_ATTRIBUTES_USER               type ULONG value('FF000000'xu);


 define structure
   1 TRUSTED_DOMAIN_INFORMATION_EX,
     2 Name            type LSA_UNICODE_STRING,
     2 FlatName        type LSA_UNICODE_STRING,
     2 Sid             type PSID,
     2 TrustDirection  type ULONG,
     2 TrustType       type ULONG,
     2 TrustAttributes type ULONG;
 define alias PTRUSTED_DOMAIN_INFORMATION_EX
        handle TRUSTED_DOMAIN_INFORMATION_EX;

 /*                                                                   */
 /* Type of authentication information                                */
 /*                                                                   */
 dcl TRUST_AUTH_TYPE_NONE    fixed bin(31) value(0);
     /* Ignore this entry */
 dcl TRUST_AUTH_TYPE_NT4OWF  fixed bin(31) value(1);
     /* NT4 OWF password */
 dcl TRUST_AUTH_TYPE_CLEAR   fixed bin(31) value(2);
     /* Cleartext password */
 dcl TRUST_AUTH_TYPE_VERSION fixed bin(31) value(3);
     /* Cleartext password version number */

 define structure
   1 LSA_AUTH_INFORMATION,
     2 LastUpdateTime type LARGE_INTEGER,
     2 AuthType       type ULONG,
     2 AuthInfoLength type ULONG,
     2 AuthInfo       type PUCHAR;
 define alias PLSA_AUTH_INFORMATION handle LSA_AUTH_INFORMATION;

 define structure
   1 TRUSTED_DOMAIN_AUTH_INFORMATION,
     2 IncomingAuthInfos                 type ULONG,
     2 IncomingAuthenticationInformation type PLSA_AUTH_INFORMATION,
     2 IncomingPreviousAuthenticationInformation
                                         type PLSA_AUTH_INFORMATION,
     2 OutgoingAuthInfos                 type ULONG,
     2 OutgoingAuthenticationInformation type PLSA_AUTH_INFORMATION,
     2 OutgoingPreviousAuthenticationInformation
                                         type PLSA_AUTH_INFORMATION;
 define alias PTRUSTED_DOMAIN_AUTH_INFORMATION
        handle TRUSTED_DOMAIN_AUTH_INFORMATION;

 define structure
   1 TRUSTED_DOMAIN_FULL_INFORMATION,
     2 Information     type TRUSTED_DOMAIN_INFORMATION_EX,
     2 PosixOffset     type TRUSTED_POSIX_OFFSET_INFO,
     2 AuthInformation type TRUSTED_DOMAIN_AUTH_INFORMATION;
 define alias PTRUSTED_DOMAIN_FULL_INFORMATION
        handle TRUSTED_DOMAIN_FULL_INFORMATION;



 /*                                                                   */
 /* LSA Enumeration Context                                           */
 /*                                                                   */

 define alias  LSA_ENUMERATION_HANDLE type ULONG;
 define alias PLSA_ENUMERATION_HANDLE pointer; /* to ULONG*/

 /*                                                                   */
 /* LSA Enumeration Information                                       */
 /*                                                                   */

 define structure
   1 LSA_ENUMERATION_INFORMATION,
     2 Sid type PSID;
 define alias PLSA_ENUMERATION_INFORMATION
        handle LSA_ENUMERATION_INFORMATION;


 /*********************************************************************/
 /*                                                                   */
 /* Local Security Policy - Miscellaneous API function prototypes     */
 /*                                                                   */
 /*********************************************************************/


 dcl LsaFreeMemory
     entry (type PVOID)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaFreeMemory');

 dcl LsaClose
     entry (type LSA_HANDLE)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaClose');

 dcl LsaOpenPolicy
     entry (type PLSA_UNICODE_STRING byaddr nonasgn optional,
            type PLSA_OBJECT_ATTRIBUTES byaddr nonasgn,
            type ACCESS_MASK,
            type LSA_HANDLE byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaOpenPolicy');

 dcl LsaQueryInformationPolicy
     entry (type LSA_HANDLE,
            type POLICY_INFORMATION_CLASS,
            type PVOID byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaQueryInformationPolicy');

 dcl LsaSetInformationPolicy
     entry (type LSA_HANDLE,
            type POLICY_INFORMATION_CLASS,
            type PVOID)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaSetInformationPolicy');

 dcl LsaQueryDomainInformationPolicy
     entry (type LSA_HANDLE,
            type POLICY_DOMAIN_INFORMATION_CLASS,
            type PVOID byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaQueryDomainInformationPolicy');

 dcl LsaSetDomainInformationPolicy
     entry (type LSA_HANDLE,
            type POLICY_DOMAIN_INFORMATION_CLASS,
            type PVOID)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaSetDomainInformationPolicy');


 dcl LsaRegisterPolicyChangeNotification
     entry (type POLICY_NOTIFICATION_INFORMATION_CLASS,
            type HANDLE)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaRegisterPolicyChangeNotification');

 dcl LsaUnregisterPolicyChangeNotification
     entry (type POLICY_NOTIFICATION_INFORMATION_CLASS,
            type HANDLE)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaUnregisterPolicyChangeNotification');


 dcl LsaEnumerateTrustedDomains
     entry (type LSA_HANDLE,
            type LSA_ENUMERATION_HANDLE byaddr,
            type PVOID byaddr,
            type ULONG,
            type ULONG byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaEnumerateTrustedDomains');


 dcl LsaLookupNames
     entry (type LSA_HANDLE,
            type ULONG,
            (*) type LSA_UNICODE_STRING byaddr nonasgn,
            type PLSA_REFERENCED_DOMAIN_LIST byaddr,
            type PLSA_TRANSLATED_SID byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaLookupNames');

 dcl LsaLookupSids
     entry (type LSA_HANDLE,
            type ULONG,
            (*) type PSID byaddr nonasgn,
            type PLSA_REFERENCED_DOMAIN_LIST byaddr,
            type PLSA_TRANSLATED_NAME byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaLookupSids');



 dcl SE_INTERACTIVE_LOGON_NAME  char value("SeInteractiveLogonRight");
 dcl SE_NETWORK_LOGON_NAME      char value("SeNetworkLogonRight");
 dcl SE_BATCH_LOGON_NAME        char value("SeBatchLogonRight");
 dcl SE_SERVICE_LOGON_NAME      char value("SeServiceLogonRight");
 dcl SE_DENY_INTERACTIVE_LOGON_NAME
                              char value("SeDenyInteractiveLogonRight");
 dcl SE_DENY_NETWORK_LOGON_NAME char value("SeDenyNetworkLogonRight");
 dcl SE_DENY_BATCH_LOGON_NAME   char value("SeDenyBatchLogonRight");
 dcl SE_DENY_SERVICE_LOGON_NAME char value("SeDenyServiceLogonRight");

 /*                                                                   */
 /* This new API returns all the accounts with a certain privilege    */
 /*                                                                   */

 dcl LsaEnumerateAccountsWithUserRight
     entry (type LSA_HANDLE,
            type LSA_UNICODE_STRING byaddr nonasgn optional,
            type PVOID byaddr,
            type ULONG byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaEnumerateAccountsWithUserRight');

 /*                                                                   */
 /* These new APIs differ by taking a SID instead of requiring the    */
 /* caller to open the account first and passing in an account handle */
 /*                                                                   */

 dcl LsaEnumerateAccountRights
     entry (type LSA_HANDLE,
            type PSID,
            type PLSA_UNICODE_STRING byaddr,
            type ULONG byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaEnumerateAccountRights');

 dcl LsaAddAccountRights
     entry (type LSA_HANDLE,
            type PSID,
            (*) type LSA_UNICODE_STRING byaddr nonasgn,
            type ULONG)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaAddAccountRights');

 dcl LsaRemoveAccountRights
     entry (type LSA_HANDLE,
            type PSID,
            type BOOLEAN,
            (*) type LSA_UNICODE_STRING byaddr nonasgn,
            type ULONG)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaRemoveAccountRights');


 /*********************************************************************/
 /*                                                                   */
 /* Local Security Policy - Trusted Domain Object API function        */
 /*                         prototypes                                */
 /*                                                                   */
 /*********************************************************************/

 dcl LsaOpenTrustedDomainByName
     entry (type LSA_HANDLE,
            type LSA_UNICODE_STRING byaddr nonasgn,
            type ACCESS_MASK,
            type LSA_HANDLE byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaOpenTrustedDomainByName');


 dcl LsaQueryTrustedDomainInfo
     entry (type LSA_HANDLE,
            type PSID,
            type TRUSTED_INFORMATION_CLASS,
            type PVOID byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaQueryTrustedDomainInfo');

 dcl LsaSetTrustedDomainInformation
     entry (type LSA_HANDLE,
            type PSID,
            type TRUSTED_INFORMATION_CLASS,
            type PVOID)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaSetTrustedDomainInformation');

 dcl LsaDeleteTrustedDomain
     entry (type LSA_HANDLE,
            type PSID)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaDeleteTrustedDomain');

 dcl LsaQueryTrustedDomainInfoByName
     entry (type LSA_HANDLE,
            type LSA_UNICODE_STRING byaddr nonasgn,
            type TRUSTED_INFORMATION_CLASS,
            type PVOID byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaQueryTrustedDomainInfoByName');

 dcl LsaSetTrustedDomainInfoByName
     entry (type LSA_HANDLE,
            type LSA_UNICODE_STRING byaddr nonasgn,
            type TRUSTED_INFORMATION_CLASS,
            type PVOID)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaSetTrustedDomainInfoByName');

 dcl LsaEnumerateTrustedDomainsEx
     entry (type LSA_HANDLE,
            type LSA_ENUMERATION_HANDLE byaddr,
            type PVOID byaddr,
            type ULONG,
            type ULONG byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaEnumerateTrustedDomainsEx');

 dcl LsaCreateTrustedDomainEx
     entry (type LSA_HANDLE,
            type TRUSTED_DOMAIN_INFORMATION_EX byaddr nonasgn,
            type TRUSTED_DOMAIN_AUTH_INFORMATION byaddr nonasgn,
            type ACCESS_MASK,
            type LSA_HANDLE byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaCreateTrustedDomainEx');


 /*                                                                   */
 /* This API sets the workstation password (equivalent of             */
 /* setting/getting the SSI_SECRET_NAME secret)                       */
 /*                                                                   */

 dcl LsaStorePrivateData
     entry (type LSA_HANDLE,
            type LSA_UNICODE_STRING byaddr nonasgn,
            type LSA_UNICODE_STRING byaddr nonasgn optional)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaStorePrivateData');

 dcl LsaRetrievePrivateData
     entry (type LSA_HANDLE,
            type LSA_UNICODE_STRING byaddr nonasgn,
            type PLSA_UNICODE_STRING byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaRetrievePrivateData');


 dcl LsaNtStatusToWinError
     entry (type NTSTATUS)
     returns (type ULONG byvalue optional)
     NTAPI external('LsaNtStatusToWinError');

 %if 0 ^= 0 %then %do;
 dcl LsaLookupNamesEx
     entry (type LSA_HANDLE,
            type ULONG,
            type LSA_NAME_LOOKUP_EX byaddr,
            type PLSA_TRANSLATED_SID_EX byaddr,
            type ULONG,
            type ULONG byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaLookupNamesEx');

 dcl LsaLookupSidsEx
     entry (type LSA_HANDLE,
            type ULONG,
            type LSA_SID_LOOKUP_EX byaddr,
            type PLSA_TRANSLATED_NAME_EX byaddr,
            type ULONG,
            type ULONG byaddr)
     returns (type NTSTATUS byvalue optional)
     NTAPI external('LsaLookupSidsEx');
 %end;

 /*                                                                   */
 /* Define a symbol so we can tell if ntifs.cpy has been included.    */
 /*                                                                   */

 /* begin_ntifs */
 %dcl #NTLSA_IFS_ char ext;
 %if #NTLSA_IFS_ = '' %then %do;
 %#NTLSA_IFS_ = 'Y';
 %end;
 /* end_ntifs */


 /*                                                                   */
 /* SPNEGO package stuff                                              */
 /*                                                                   */

 define ordinal
   NEGOTIATE_MESSAGES
     ( NegEnumPackagePrefixes value(0),
       NegGetCallerName       value(1),
       NegCallPackageMax);

 dcl NEGOTIATE_MAX_PREFIX fixed bin(31) value(32);

 define structure
   1 NEGOTIATE_PACKAGE_PREFIX,
     2 PackageId    type ULONG_PTR,
     2 PackageDataA type PVOID,
     2 PackageDataW type PVOID,
     2 PrefixLen    type ULONG_PTR,
     2 Prefix       (NEGOTIATE_MAX_PREFIX) type UCHAR;
 define alias PNEGOTIATE_PACKAGE_PREFIX handle NEGOTIATE_PACKAGE_PREFIX;

 define structure
   1 NEGOTIATE_PACKAGE_PREFIXES,
     2 MessageType  type ULONG,
     2 PrefixCount  type ULONG,
     2 Offset       type ULONG;/* Offset to array of _PREFIX above */
 define alias PNEGOTIATE_PACKAGE_PREFIXES
        handle NEGOTIATE_PACKAGE_PREFIXES;

 define structure
   1 NEGOTIATE_CALLER_NAME_REQUEST,
     2 MessageType  type ULONG,
     2 LogonId      type LUID;
 define alias PNEGOTIATE_CALLER_NAME_REQUEST
        handle NEGOTIATE_CALLER_NAME_REQUEST;

 define structure
   1 NEGOTIATE_CALLER_NAME_RESPONSE,
     2 MessageType type ULONG,
     2 CallerName  type PWSTR;
 define alias PNEGOTIATE_CALLER_NAME_RESPONSE
        handle NEGOTIATE_CALLER_NAME_RESPONSE;

 dcl NEGOTIATE_ALLOW_NTLM type ULONG value('10000000'xu);
 dcl NEGOTIATE_NEG_NTLM   type ULONG value('20000000'xu);

 %dcl #NTDEF_ char ext;
 %if #NTDEF_ = '' %then %do;
 /* already defined in subauth, included above */
 /* define alias PUNICODE_STRING handle LSA_UNICODE_STRING */
 /* define alias PSTRING handle LSA_STRING */
 %end;

 %dcl #DOMAIN_PASSWORD_INFORMATION_DEFINED char ext;
 %if #DOMAIN_PASSWORD_INFORMATION_DEFINED = '' %then %do;
 %#DOMAIN_PASSWORD_INFORMATION_DEFINED = 'Y';
 define structure
   1 DOMAIN_PASSWORD_INFORMATION,
     2 MinPasswordLength     type USHORT,
     2 PasswordHistoryLength type USHORT,
     2 PasswordProperties    type ULONG,
     2 MaxPasswordAge        type LARGE_INTEGER,
     2 MinPasswordAge        type LARGE_INTEGER;
 define alias PDOMAIN_PASSWORD_INFORMATION
        handle DOMAIN_PASSWORD_INFORMATION;
 %end;


 %dcl #PASSWORD_NOTIFICATION_DEFINED char ext;
 %if #PASSWORD_NOTIFICATION_DEFINED = '' %then %do;
 %#PASSWORD_NOTIFICATION_DEFINED = 'Y';
 define alias PSAM_PASSWORD_NOTIFICATION_ROUTINE
        entry (type UNICODE_STRING byaddr,
               type ULONG,
               type UNICODE_STRING byaddr)
        returns (type NTSTATUS byvalue optional)
        options (byvalue nodescriptor) limited;

 dcl SAM_PASSWORD_CHANGE_NOTIFY_ROUTINE
     char value('PasswordChangeNotify');

 define alias PSAM_INIT_NOTIFICATION_ROUTINE
        entry ()
        returns (type BOOLEAN byvalue optional)
        options (byvalue nodescriptor) limited;

 dcl SAM_INIT_NOTIFICATION_ROUTINE char value('InitializeChangeNotify');


 dcl SAM_PASSWORD_FILTER_ROUTINE char value('PasswordFilter');

 define alias PSAM_PASSWORD_FILTER_ROUTINE
        entry (type UNICODE_STRING byaddr,
               type UNICODE_STRING byaddr,
               type UNICODE_STRING byaddr,
               type BOOLEAN)
        returns (type BOOLEAN byvalue optional)
        options (byvalue nodescriptor) limited;
 %end; /* _PASSWORD_NOTIFICATION_DEFINED */

 /*********************************************************************/
 /*                                                                   */
 /* Name of the MSV1_0 authentication package                         */
 /*                                                                   */
 /*********************************************************************/

 dcl MSV1_0_PACKAGE_NAME  char
     value('MICROSOFT_AUTHENTICATION_PACKAGE_V1_0');
 dcl MSV1_0_PACKAGE_NAMEW WCHAR
     value('MICROSOFT_AUTHENTICATION_PACKAGE_V1_0');
 dcl MSV1_0_PACKAGE_NAMEW_LENGTH fixed bin(31)
     value(2 * (length (MSV1_0_PACKAGE_NAMEW) - 1)); /* what else? */

 /*                                                                   */
 /* Location of MSV authentication package data                       */
 /*                                                                   */
 dcl MSV1_0_SUBAUTHENTICATION_KEY   char
     value('SYSTEM\CurrentControlSet\Control\Lsa\MSV1_0');
 dcl MSV1_0_SUBAUTHENTICATION_VALUE char value('Auth');


 /*********************************************************************/
 /*                                                                   */
 /* Widely used MSV1_0 data types                                     */
 /*                                                                   */
 /*********************************************************************/


 /*********************************************************************/
 /*                                                                   */
 /*       LOGON      Related Data Structures                          */
 /*                                                                   */
 /*                                                                   */
 /*********************************************************************/

 /*                                                                   */
 /* When a LsaLogonUser() call is dispatched to the MsV1_0            */
 /* authentication package, the beginning of the                      */
 /* AuthenticationInformation buffer is cast to a                     */
 /* MSV1_0_LOGON_SUBMIT_TYPE to determine the type of logon being     */
 /* requested.  Similarly, upon return, the type of profile buffer    */
 /* can be determined by typecasting it to a                          */
 /* MSV_1_0_PROFILE_BUFFER_TYPE.                                      */
 /*                                                                   */

 /*                                                                   */
 /*  MSV1.0 LsaLogonUser() submission message types.                  */
 /*                                                                   */

 define ordinal
   MSV1_0_LOGON_SUBMIT_TYPE
     ( MsV1_0InteractiveLogon value(2),
       MsV1_0Lm20Logon,
       MsV1_0NetworkLogon,
       MsV1_0SubAuthLogon,
       MsV1_0WorkstationUnlockLogon value(7));
 dcl PMSV1_0_LOGON_SUBMIT_TYPE pointer; /* to MSV1_0_LOGON_SUBMIT_TYPE*/


 /*                                                                   */
 /*  MSV1.0 LsaLogonUser() profile buffer types.                      */
 /*                                                                   */

 define ordinal
   MSV1_0_PROFILE_BUFFER_TYPE
     ( MsV1_0InteractiveProfile value(2),
       MsV1_0Lm20LogonProfile,
       MsV1_0SmartCardProfile);
 dcl PMSV1_0_PROFILE_BUFFER_TYPE pointer;
     /* to MSV1_0_PROFILE_BUFFER_TYPE */


 /*                                                                   */
 /* MsV1_0InteractiveLogon                                            */
 /*                                                                   */
 /* The AuthenticationInformation buffer of an LsaLogonUser() call    */
 /* to perform an interactive logon contains the following data       */
 /* structure:                                                        */
 /*                                                                   */

 define structure
   1 MSV1_0_INTERACTIVE_LOGON,
     2 MessageType     type MSV1_0_LOGON_SUBMIT_TYPE,
     2 LogonDomainName type UNICODE_STRING,
     2 UserName        type UNICODE_STRING,
     2 Password        type UNICODE_STRING;
 define alias PMSV1_0_INTERACTIVE_LOGON handle MSV1_0_INTERACTIVE_LOGON;

 /*                                                                   */
 /* Where:                                                            */
 /*                                                                   */
 /*     MessageType - Contains the type of logon being requested.     */
 /*         This field must be set to MsV1_0InteractiveLogon.         */
 /*                                                                   */
 /*     UserName - Is a string representing the user's account name.  */
 /*         The name may be up to 255 characters long.                */
 /*         The name is treated case insensitive.                     */
 /*                                                                   */
 /*     Password - Is a string containing the user's cleartext        */
 /*         password. The password may be up to 255 characters long   */
 /*         and contain any UNICODE value.                            */
 /*                                                                   */
 /*                                                                   */


 /*                                                                   */
 /* The ProfileBuffer returned upon a successful logon of this type   */
 /* contains the following data structure:                            */
 /*                                                                   */

 define structure
   1 MSV1_0_INTERACTIVE_PROFILE,
     2 MessageType        type MSV1_0_PROFILE_BUFFER_TYPE,
     2 LogonCount         type USHORT,
     2 BadPasswordCount   type USHORT,
     2 LogonTime          type LARGE_INTEGER,
     2 LogoffTime         type LARGE_INTEGER,
     2 KickOffTime        type LARGE_INTEGER,
     2 PasswordLastSet    type LARGE_INTEGER,
     2 PasswordCanChange  type LARGE_INTEGER,
     2 PasswordMustChange type LARGE_INTEGER,
     2 LogonScript        type UNICODE_STRING,
     2 HomeDirectory      type UNICODE_STRING,
     2 FullName           type UNICODE_STRING,
     2 ProfilePath        type UNICODE_STRING,
     2 HomeDirectoryDrive type UNICODE_STRING,
     2 LogonServer        type UNICODE_STRING,
     2 UserFlags          type ULONG;
 define alias PMSV1_0_INTERACTIVE_PROFILE
        handle MSV1_0_INTERACTIVE_PROFILE;

 /*                                                                   */
 /* where:                                                            */
 /*                                                                   */
 /*     MessageType - Identifies the type of profile data being       */
 /*         returned. Contains the type of logon being requested.     */
 /*         This field must be set to MsV1_0InteractiveProfile.       */
 /*                                                                   */
 /*     LogonCount - Number of times the user is currently logged on. */
 /*                                                                   */
 /*     BadPasswordCount - Number of times a bad password was applied */
 /*         to the account since last successful logon.               */
 /*                                                                   */
 /*     LogonTime - Time when user last logged on.                    */
 /*         This is an absolute format NT standard time value.        */
 /*                                                                   */
 /*     LogoffTime - Time when user should log off.                   */
 /*         This is an absolute format NT standard time value.        */
 /*                                                                   */
 /*     KickOffTime - Time when system should force user logoff.      */
 /*         This is an absolute format NT standard time value.        */
 /*                                                                   */
 /*     PasswordLastChanged - Time and date the password was last     */
 /*         changed.  This is an absolute format NT standard time     */
 /*         value.                                                    */
 /*                                                                   */
 /*     PasswordCanChange - Time and date when the user can change    */
 /*         the password.  This is an absolute format NT time value.  */
 /*         To prevent a password from ever changing, set this field  */
 /*         to a date very far into the future.                       */
 /*                                                                   */
 /*     PasswordMustChange - Time and date when the user must change  */
 /*         the password.  If the user can never change the password, */
 /*         this field is undefined.  This is an absolute format NT   */
 /*         time value.                                               */
 /*                                                                   */
 /*     LogonScript - The (relative) path to the account's logon      */
 /*         script.                                                   */
 /*                                                                   */
 /*     HomeDirectory - The home directory for the user.              */
 /*                                                                   */


 /*                                                                   */
 /* MsV1_0Lm20Logon and MsV1_0NetworkLogon                            */
 /*                                                                   */
 /* The AuthenticationInformation buffer of an LsaLogonUser() call to */
 /* perform an network logon contains the following data structure:   */
 /*                                                                   */
 /* MsV1_0NetworkLogon logon differs from MsV1_0Lm20Logon in that the */
 /* ParameterControl field exists.                                    */
 /*                                                                   */

 dcl MSV1_0_CHALLENGE_LENGTH                fixed bin(31) value(8);
 dcl MSV1_0_USER_SESSION_KEY_LENGTH         fixed bin(31) value(16);
 dcl MSV1_0_LANMAN_SESSION_KEY_LENGTH       fixed bin(31) value(8);



 /*                                                                   */
 /* Values for ParameterControl.                                      */
 /*                                                                   */

 dcl MSV1_0_CLEARTEXT_PASSWORD_ALLOWED      type ULONG value('02'xu);
 dcl MSV1_0_UPDATE_LOGON_STATISTICS         type ULONG value('04'xu);
 dcl MSV1_0_RETURN_USER_PARAMETERS          type ULONG value('08'xu);
 dcl MSV1_0_DONT_TRY_GUEST_ACCOUNT          type ULONG value('10'xu);
 dcl MSV1_0_ALLOW_SERVER_TRUST_ACCOUNT      type ULONG value('20'xu);
 dcl MSV1_0_RETURN_PASSWORD_EXPIRY          type ULONG value('40'xu);
 /* this next flag says that CaseInsensitiveChallengeResponse */
 /*  (aka LmResponse) contains a client challenge in the first 8 bytes
 */
 dcl MSV1_0_USE_CLIENT_CHALLENGE            type ULONG value('80'xu);
 dcl MSV1_0_TRY_GUEST_ACCOUNT_ONLY          type ULONG value('100'xu);
 dcl MSV1_0_RETURN_PROFILE_PATH             type ULONG value('200'xu);
 dcl MSV1_0_TRY_SPECIFIED_DOMAIN_ONLY       type ULONG value('400'xu);
 dcl MSV1_0_ALLOW_WORKSTATION_TRUST_ACCOUNT type ULONG value('800'xu);
 dcl MSV1_0_SUBAUTHENTICATION_DLL_EX    type ULONG value('00100000'xu);

 /*                                                                   */
 /* The high order byte is a value indicating the SubAuthentication   */
 /* DLL. Zero indicates no SubAuthentication DLL.                     */
 /*                                                                   */
 dcl MSV1_0_SUBAUTHENTICATION_DLL       type ULONG  value('FF000000'xu);
 dcl MSV1_0_SUBAUTHENTICATION_DLL_SHIFT fixed bin(31) value(24);
 dcl MSV1_0_MNS_LOGON                   type ULONG  value('01000000'xu);

 /*                                                                   */
 /* This is the list of subauthentication dlls used in MS             */
 /*                                                                   */

 dcl MSV1_0_SUBAUTHENTICATION_DLL_RAS fixed bin(31) value(2);
 dcl MSV1_0_SUBAUTHENTICATION_DLL_IIS fixed bin(31) value(132);

 define structure
   1 MSV1_0_LM20_LOGON,
     2 MessageType      type MSV1_0_LOGON_SUBMIT_TYPE,
     2 LogonDomainName  type UNICODE_STRING,
     2 UserName         type UNICODE_STRING,
     2 Workstation      type UNICODE_STRING,
     2 ChallengeToClient (MSV1_0_CHALLENGE_LENGTH) type UCHAR,
     2 CaseSensitiveChallengeResponse   type STRING,
     2 CaseInsensitiveChallengeResponse type STRING,
     2 ParameterControl                 type ULONG;

 define alias PMSV1_0_LM20_LOGON handle MSV1_0_LM20_LOGON;



 /*                                                                   */
 /* NT 5.0 SubAuth dlls can use this struct                           */
 /*                                                                   */

 define structure
   1 MSV1_0_SUBAUTH_LOGON,
     2 MessageType         type MSV1_0_LOGON_SUBMIT_TYPE,
     2 LogonDomainName     type UNICODE_STRING,
     2 UserName            type UNICODE_STRING,
     2 Workstation         type UNICODE_STRING,
     2 ChallengeToClient (MSV1_0_CHALLENGE_LENGTH) type UCHAR,
     2 AuthenticationInfo1 type STRING,
     2 AuthenticationInfo2 type STRING,
     2 ParameterControl    type ULONG,
     2 SubAuthPackageId    type ULONG;

 define alias PMSV1_0_SUBAUTH_LOGON handle MSV1_0_SUBAUTH_LOGON;


 /*                                                                   */
 /* Values for UserFlags.                                             */
 /*                                                                   */

 dcl LOGON_GUEST                    type ULONG value('01'xu);
 dcl LOGON_NOENCRYPTION             type ULONG value('02'xu);
 dcl LOGON_CACHED_ACCOUNT           type ULONG value('04'xu);
 dcl LOGON_USED_LM_PASSWORD         type ULONG value('08'xu);
 dcl LOGON_EXTRA_SIDS               type ULONG value('20'xu);
 dcl LOGON_SUBAUTH_SESSION_KEY      type ULONG value('40'xu);
 dcl LOGON_SERVER_TRUST_ACCOUNT     type ULONG value('80'xu);
 dcl LOGON_NTLMV2_ENABLED           type ULONG value('100'xu);
     /* says DC understands NTLMv2 */
 dcl LOGON_RESOURCE_GROUPS          type ULONG value('200'xu);
 dcl LOGON_PROFILE_PATH_RETURNED    type ULONG value('400'xu);

 /*                                                                   */
 /* The high order byte is reserved for return by SubAuthentication   */
 /* DLLs.                                                             */
 /*                                                                   */

 dcl MSV1_0_SUBAUTHENTICATION_FLAGS type ULONG value('FF000000'xu);

 /* Values returned by the MSV1_0_MNS_LOGON SubAuthentication DLL */
 dcl LOGON_GRACE_LOGON              type ULONG value('01000000'xu);

 define structure
   1 MSV1_0_LM20_LOGON_PROFILE,
     2 MessageType      type MSV1_0_PROFILE_BUFFER_TYPE,
     2 KickOffTime      type LARGE_INTEGER,
     2 LogoffTime       type LARGE_INTEGER,
     2 UserFlags        type ULONG,
     2 UserSessionKey   (MSV1_0_USER_SESSION_KEY_LENGTH) type UCHAR,
     2 LogonDomainName  type UNICODE_STRING,
     2 LanmanSessionKey (MSV1_0_LANMAN_SESSION_KEY_LENGTH) type UCHAR,
     2 LogonServer      type UNICODE_STRING,
     2 UserParameters   type UNICODE_STRING;

 define alias PMSV1_0_LM20_LOGON_PROFILE
        handle MSV1_0_LM20_LOGON_PROFILE;


 /*                                                                   */
 /* Supplemental credentials structure used for passing credentials   */
 /* into MSV1_0 from other packages                                   */
 /*                                                                   */

 dcl MSV1_0_OWF_PASSWORD_LENGTH fixed bin(31) value(16);
 dcl MSV1_0_CRED_LM_PRESENT     type ULONG value('1'xu);
 dcl MSV1_0_CRED_NT_PRESENT     type ULONG value('2'xu);
 dcl MSV1_0_CRED_VERSION        fixed bin(31) value(0);

 define structure
   1 MSV1_0_SUPPLEMENTAL_CREDENTIAL,
     2 Version    type ULONG,
     2 Flags      type ULONG,
     2 LmPassword (MSV1_0_OWF_PASSWORD_LENGTH) type UCHAR,
     2 NtPassword (MSV1_0_OWF_PASSWORD_LENGTH) type UCHAR;
 define alias PMSV1_0_SUPPLEMENTAL_CREDENTIAL
        handle MSV1_0_SUPPLEMENTAL_CREDENTIAL;


 /*                                                                   */
 /* NTLM3 definitions.                                                */
 /*                                                                   */

 dcl MSV1_0_NTLM3_RESPONSE_LENGTH fixed bin(31) value(16);
 dcl MSV1_0_NTLM3_OWF_LENGTH      fixed bin(31) value(16);

 /*                                                                   */
 /* this is the longest amount of time we'll allow challenge response */
 /* pairs to be used. Note that this also has to allow for worst case */
 /* clock skew                                                        */
 /*                                                                   */
 dcl MSV1_0_MAX_NTLM3_LIFE fixed bin(31) value(1800);
     /* 30 minutes (in seconds) */
 dcl MSV1_0_MAX_AVL_SIZE   fixed bin(31) value(64000);

 /* this is an MSV1_0 private data structure, defining the layout of  */
 /* an NTLM3 response, as sent by a client in the NtChallengeResponse */
 /* field of the NETLOGON_NETWORK_INFO structure.                     */
 /* If can be differentiated from an old style NT response by its     */
 /* length. This is crude, but it needs to pass through servers and   */
 /* the servers' DCs that do not understand NTLM3 but that are        */
 /* willing to pass longer responses.                                 */

 define structure
   1 MSV1_0_NTLM3_RESPONSE,
     2 Response (MSV1_0_NTLM3_RESPONSE_LENGTH) type UCHAR,
       /* hash of OWF of password with all the following fields */
     2 RespType   type UCHAR,
       /* id number of response; current is 1 */
     2 HiRespType type UCHAR,
       /* highest id number understood by client */
     2 Flags      type USHORT,
       /* reserved; must be sent as zero at this version */
     2 MsgWord    type ULONG,
       /* 32 bit message from client to server */
       /* (for use by auth protocol) */
     2 TimeStamp  type ULONGLONG,
       /* time stamp when client generated response */
       /* -- NT system time, quad part              */
     2 ChallengeFromClient (MSV1_0_CHALLENGE_LENGTH) type UCHAR,
     2 AvPairsOff type ULONG,
       /* offset to start of AvPairs (to allow future expansion) */
     2 Buffer(1)  type UCHAR;
       /* start of buffer with AV pairs */
       /* (or future stuff -- so use the offset) */

 dcl PMSV1_0_NTLM3_RESPONSE handle MSV1_0_NTLM3_RESPONSE;

 dcl MSV1_0_NTLM3_INPUT_LENGTH fixed bin(31)
     value (size (: MSV1_0_NTLM3_RESPONSE :)
            - MSV1_0_NTLM3_RESPONSE_LENGTH);

 define ordinal
   MSV1_0_AVID
     ( MsvAvEOL,            /* end of list                       */
       MsvAvNbComputerName, /* server's computer name -- NetBIOS */
       MsvAvNbDomainName,   /* server's domain name   -- NetBIOS */
       MsvAvDnsComputerName,/* server's computer name -- DNS     */
       MsvAvDnsDomainName); /* server's domain name   -- DNS     */

 define structure
   1 MSV1_0_AV_PAIR,
     2 AvId  type USHORT,
     2 AvLen type USHORT;
       /* Data is treated as byte array following structure */
 define alias PMSV1_0_AV_PAIR handle MSV1_0_AV_PAIR;


 /*********************************************************************/
 /*                                                                   */
 /*       CALL PACKAGE Related Data Structures                        */
 /*                                                                   */
 /*********************************************************************/


 /*                                                                   */
 /*  MSV1.0 LsaCallAuthenticationPackage() submission and response    */
 /*  message types.                                                   */
 /*                                                                   */

 define ordinal
   MSV1_0_PROTOCOL_MESSAGE_TYPE
     ( MsV1_0Lm20ChallengeRequest value(0),
                                      /* Both submission and response */
       MsV1_0Lm20GetChallengeResponse,/* Both submission and response */
       MsV1_0EnumerateUsers,          /* Both submission and response */
       MsV1_0GetUserInfo,             /* Both submission and response */
       MsV1_0ReLogonUsers,            /* Submission only */
       MsV1_0ChangePassword,          /* Both submission and response */
       MsV1_0ChangeCachedPassword,    /* Both submission and response */
       MsV1_0GenericPassthrough,      /* Both submission and response */
       MsV1_0CacheLogon,              /* Submission only, no response */
       MsV1_0SubAuth,                 /* Both submission and response */
       MsV1_0DeriveCredential,        /* Both submission and response */
       MsV1_0CacheLookup);            /* Both submission and response */
 dcl PMSV1_0_PROTOCOL_MESSAGE_TYPE pointer;
     /* to #MSV1_0_PROTOCOL_MESSAGE_TYPE */


 define structure
   1 MSV1_0_CHANGEPASSWORD_REQUEST,
     2 MessageType   type MSV1_0_PROTOCOL_MESSAGE_TYPE,
     2 DomainName    type UNICODE_STRING,
     2 AccountName   type UNICODE_STRING,
     2 OldPassword   type UNICODE_STRING,
     2 NewPassword   type UNICODE_STRING,
     2 Impersonating type BOOLEAN;

 define alias PMSV1_0_CHANGEPASSWORD_REQUEST
        handle MSV1_0_CHANGEPASSWORD_REQUEST;

 define structure
   1 MSV1_0_CHANGEPASSWORD_RESPONSE,
     2 MessageType        type MSV1_0_PROTOCOL_MESSAGE_TYPE,
     2 PasswordInfoValid  type BOOLEAN,
     2 DomainPasswordInfo type DOMAIN_PASSWORD_INFORMATION;

 define alias PMSV1_0_CHANGEPASSWORD_RESPONSE
        handle MSV1_0_CHANGEPASSWORD_RESPONSE;


 /*                                                                   */
 /* MsV1_0SubAuthInfo submit buffer and response - for submitting a   */
 /* buffer to a specified Subauthentication Package during an         */
 /* LsaCallAuthenticationPackage().                                   */
 /* If this Subauthentication is to be done locally, then package     */
 /* this message in LsaCallAuthenticationPackage().                   */
 /* If this SubAuthentication needs to be done on the domain          */
 /* controller, then call LsaCallauthenticationPackage with the       */
 /* message type being MsV1_0GenericPassThrough and the LogonData in  */
 /* this struct should be a PMSV1_0_SUBAUTH_REQUEST                   */
 /*                                                                   */

 define structure
   1 MSV1_0_SUBAUTH_REQUEST,
     2 MessageType         type MSV1_0_PROTOCOL_MESSAGE_TYPE,
     2 SubAuthPackageId    type ULONG,
     2 SubAuthInfoLength   type ULONG,
     2 SubAuthSubmitBuffer type PUCHAR;

 define alias PMSV1_0_SUBAUTH_REQUEST handle MSV1_0_SUBAUTH_REQUEST;

 define structure
   1 MSV1_0_SUBAUTH_RESPONSE,
     2 MessageType         type MSV1_0_PROTOCOL_MESSAGE_TYPE,
     2 SubAuthInfoLength   type ULONG,
     2 SubAuthReturnBuffer type PUCHAR;

 define alias PMSV1_0_SUBAUTH_RESPONSE handle MSV1_0_SUBAUTH_RESPONSE;


 /*                                                                   */
 /* Credential Derivation types for MsV1_0DeriveCredential Submit     */
 /* DeriveCredType                                                    */
 /*                                                                   */

 /*                                                                   */
 /* Derive Credential using SHA-1 and Request buffer                  */
 /* DeriveCredSubmitBuffer of length DeriveCredInfoLength mixing      */
 /* bytes. Response buffer DeriveCredReturnBuffer will contain SHA-1  */
 /* hash of size A_SHA_DIGEST_LEN (20)                                */
 /*                                                                   */

 dcl MSV1_0_DERIVECRED_TYPE_SHA1 fixed bin(31) value(0);

 /*                                                                   */
 /* MsV1_0DeriveCredential submit buffer and response - for submitting*/
 /* a buffer an call to LsaCallAuthenticationPackage().               */
 /*                                                                   */

 define structure
   1 MSV1_0_DERIVECRED_REQUEST,
     2 MessageType            type MSV1_0_PROTOCOL_MESSAGE_TYPE,
     2 LogonId                type LUID,
     2 DeriveCredType         type ULONG,
     2 DeriveCredInfoLength   type ULONG,
     2 DeriveCredSubmitBuffer (1) type UCHAR;
       /* in-place array of length DeriveCredInfoLength */

 define alias PMSV1_0_DERIVECRED_REQUEST
        handle MSV1_0_DERIVECRED_REQUEST;

 define structure
   1 MSV1_0_DERIVECRED_RESPONSE,
     2 MessageType            type MSV1_0_PROTOCOL_MESSAGE_TYPE,
     2 DeriveCredInfoLength   type ULONG,
     2 DeriveCredReturnBuffer (1) type UCHAR;
       /* in-place array of length DeriveCredInfoLength */

 define alias PMSV1_0_DERIVECRED_RESPONSE
        handle MSV1_0_DERIVECRED_RESPONSE;


 /* Revision of the Kerberos Protocol.  MS uses Version 5, Revision 6 */

 dcl KERBEROS_VERSION fixed bin(31) value(5);
 dcl KERBEROS_REVISION fixed bin(31) value(6);



 /* Encryption Types:                                                 */
 /* These encryption types are supported by the default MS KERBSUPP   */
 /* DLL as crypto systems.  Values over 127 are local values, and may */
 /* be changed without notice.                                        */

 dcl KERB_ETYPE_NULL              fixed bin(31) value(0);
 dcl KERB_ETYPE_DES_CBC_CRC       fixed bin(31) value(1);
 dcl KERB_ETYPE_DES_CBC_MD4       fixed bin(31) value(2);
 dcl KERB_ETYPE_DES_CBC_MD5       fixed bin(31) value(3);


 dcl KERB_ETYPE_RC4_MD4           fixed bin(31) value(-128);
 dcl KERB_ETYPE_RC4_PLAIN2        fixed bin(31) value(-129);
 dcl KERB_ETYPE_RC4_LM            fixed bin(31) value(-130);
 dcl KERB_ETYPE_RC4_SHA           fixed bin(31) value(-131);
 dcl KERB_ETYPE_DES_PLAIN         fixed bin(31) value(-132);
 dcl KERB_ETYPE_RC4_HMAC_OLD      fixed bin(31) value(-133);
 dcl KERB_ETYPE_RC4_PLAIN_OLD     fixed bin(31) value(-134);
 dcl KERB_ETYPE_RC4_HMAC_OLD_EXP  fixed bin(31) value(-135);
 dcl KERB_ETYPE_RC4_PLAIN_OLD_EXP fixed bin(31) value(-136);
 dcl KERB_ETYPE_RC4_PLAIN         fixed bin(31) value(-140);
 dcl KERB_ETYPE_RC4_PLAIN_EXP     fixed bin(31) value(-141);

 /*                                                                   */
 /* Pkinit encryption types                                           */
 /*                                                                   */


 dcl KERB_ETYPE_DSA_SHA1_CMS      fixed bin(31) value(9);
 dcl KERB_ETYPE_RSA_MD5_CMS       fixed bin(31) value(10);
 dcl KERB_ETYPE_RSA_SHA1_CMS      fixed bin(31) value(11);
 dcl KERB_ETYPE_RC2_CBC_ENV       fixed bin(31) value(12);
 dcl KERB_ETYPE_RSA_ENV           fixed bin(31) value(13);
 dcl KERB_ETYPE_RSA_ES_OEAP_ENV   fixed bin(31) value(14);
 dcl KERB_ETYPE_DES_EDE3_CBC_ENV  fixed bin(31) value(15);


 /*                                                                   */
 /* Deprecated                                                        */
 /*                                                                   */

 dcl KERB_ETYPE_DSA_SIGN          fixed bin(31) value(8);
 dcl KERB_ETYPE_RSA_PRIV          fixed bin(31) value(9);
 dcl KERB_ETYPE_RSA_PUB           fixed bin(31) value(10);
 dcl KERB_ETYPE_RSA_PUB_MD5       fixed bin(31) value(11);
 dcl KERB_ETYPE_RSA_PUB_SHA1      fixed bin(31) value(12);
 dcl KERB_ETYPE_PKCS7_PUB         fixed bin(31) value(13);

 /*                                                                   */
 /* In use types                                                      */
 /*                                                                   */

 dcl KERB_ETYPE_DES_CBC_MD5_NT    fixed bin(31) value(20);
 dcl KERB_ETYPE_RC4_HMAC_NT       fixed bin(31) value(23);
 dcl KERB_ETYPE_RC4_HMAC_NT_EXP   fixed bin(31) value(24);

 /* Checksum algorithms. */
 /* These algorithms are keyed internally for our use. */

 dcl KERB_CHECKSUM_NONE          fixed bin(31) value(0);
 dcl KERB_CHECKSUM_CRC32         fixed bin(31) value(1);
 dcl KERB_CHECKSUM_MD4           fixed bin(31) value(2);
 dcl KERB_CHECKSUM_KRB_DES_MAC   fixed bin(31) value(4);
 dcl KERB_CHECKSUM_MD5           fixed bin(31) value(7);
 dcl KERB_CHECKSUM_MD5_DES       fixed bin(31) value(8);


 dcl KERB_CHECKSUM_LM            fixed bin(31) value(-130);
 dcl KERB_CHECKSUM_SHA1          fixed bin(31) value(-131);
 dcl KERB_CHECKSUM_REAL_CRC32    fixed bin(31) value(-132);
 dcl KERB_CHECKSUM_DES_MAC       fixed bin(31) value(-133);
 dcl KERB_CHECKSUM_DES_MAC_MD5   fixed bin(31) value(-134);
 dcl KERB_CHECKSUM_MD25          fixed bin(31) value(-135);
 dcl KERB_CHECKSUM_RC4_MD5       fixed bin(31) value(-136);
 dcl KERB_CHECKSUM_MD5_HMAC      fixed bin(31) value(-137);
     /* used by netlogon */
 dcl KERB_CHECKSUM_HMAC_MD5      fixed bin(31) value(-138);
     /* used by Kerberos */

 dcl AUTH_REQ_ALLOW_FORWARDABLE     type ULONG value('00000001'xu);
 dcl AUTH_REQ_ALLOW_PROXIABLE       type ULONG value('00000002'xu);
 dcl AUTH_REQ_ALLOW_POSTDATE        type ULONG value('00000004'xu);
 dcl AUTH_REQ_ALLOW_RENEWABLE       type ULONG value('00000008'xu);
 dcl AUTH_REQ_ALLOW_NOADDRESS       type ULONG value('00000010'xu);
 dcl AUTH_REQ_ALLOW_ENC_TKT_IN_SKEY type ULONG value('00000020'xu);
 dcl AUTH_REQ_ALLOW_VALIDATE        type ULONG value('00000040'xu);
 dcl AUTH_REQ_VALIDATE_CLIENT       type ULONG value('00000080'xu);
 dcl AUTH_REQ_OK_AS_DELEGATE        type ULONG value('00000100'xu);
 dcl AUTH_REQ_PREAUTH_REQUIRED      type ULONG value('00000200'xu);


 dcl AUTH_REQ_PER_USER_FLAGS
     type ULONG value (ior (AUTH_REQ_ALLOW_FORWARDABLE,
                            AUTH_REQ_ALLOW_PROXIABLE,
                            AUTH_REQ_ALLOW_POSTDATE,
                            AUTH_REQ_ALLOW_RENEWABLE,
                            AUTH_REQ_ALLOW_VALIDATE));

 /*                                                                   */
 /* Ticket Flags:                                                     */
 /*                                                                   */

 dcl KERB_TICKET_FLAGS_reserved          type ULONG value('80000000'xu);
 dcl KERB_TICKET_FLAGS_forwardable       type ULONG value('40000000'xu);
 dcl KERB_TICKET_FLAGS_forwarded         type ULONG value('20000000'xu);
 dcl KERB_TICKET_FLAGS_proxiable         type ULONG value('10000000'xu);
 dcl KERB_TICKET_FLAGS_proxy             type ULONG value('08000000'xu);
 dcl KERB_TICKET_FLAGS_may_postdate      type ULONG value('04000000'xu);
 dcl KERB_TICKET_FLAGS_postdated         type ULONG value('02000000'xu);
 dcl KERB_TICKET_FLAGS_invalid           type ULONG value('01000000'xu);
 dcl KERB_TICKET_FLAGS_renewable         type ULONG value('00800000'xu);
 dcl KERB_TICKET_FLAGS_initial           type ULONG value('00400000'xu);
 dcl KERB_TICKET_FLAGS_pre_authent       type ULONG value('00200000'xu);
 dcl KERB_TICKET_FLAGS_hw_authent        type ULONG value('00100000'xu);
 dcl KERB_TICKET_FLAGS_ok_as_delegate    type ULONG value('00040000'xu);
 dcl KERB_TICKET_FLAGS_name_canonicalize type ULONG value('00010000'xu);
 dcl KERB_TICKET_FLAGS_reserved1         type ULONG value('00000001'xu);



 %dcl MICROSOFT_KERBEROS_NAME_DEFINED char ext;
 %if MICROSOFT_KERBEROS_NAME_DEFINED = '' %then %do;
 %MICROSOFT_KERBEROS_NAME_DEFINED = 'Y';
 dcl MICROSOFT_KERBEROS_NAME_A char  value("Kerberos");
 dcl MICROSOFT_KERBEROS_NAME_W WCHAR value("Kerberos");
 %dcl WIN32_CHICAGO char ext;
 %if WIN32_CHICAGO ^= '' %then %do;
 dcl MICROSOFT_KERBEROS_NAME char  value(MICROSOFT_KERBEROS_NAME_A);
 %end; %else %do;
 dcl MICROSOFT_KERBEROS_NAME TCHAR value(MICROSOFT_KERBEROS_NAME_W);
 %end; /* WIN32_CHICAGO */
 %end; /* MICROSOFT_KERBEROS_NAME_DEFINED */


 /*********************************************************************/
 /*                                                                   */
 /* Quality of protection parameters for MakeSignature/EncryptMessage */
 /*                                                                   */
 /*********************************************************************/

 /*                                                                   */
 /* This flag indicates to EncryptMessage that the message is not to  */
 /* actually be encrypted, but a header/trailer are to be produced.   */
 /*                                                                   */

 dcl KERB_WRAP_NO_ENCRYPT type ULONG value('80000001'xu);

 /*********************************************************************/
 /*                                                                   */
 /* LsaLogonUser parameters                                           */
 /*                                                                   */
 /*********************************************************************/

 define ordinal
   KERB_LOGON_SUBMIT_TYPE
     ( KerbInteractiveLogon       value(2),
       KerbSmartCardLogon         value(6),
       KerbWorkstationUnlockLogon value(7),
       KerbSmartCardUnlockLogon   value(8),
       KerbProxyLogon             value(9),
       KerbTicketLogon            value(10),
       KerbTicketUnlockLogon      value(11));
 dcl PKERB_LOGON_SUBMIT_TYPE pointer; /* to KERB_LOGON_SUBMIT_TYPE */


 define structure
   1 KERB_INTERACTIVE_LOGON,
     2 MessageType     type KERB_LOGON_SUBMIT_TYPE,
     2 LogonDomainName type UNICODE_STRING,
     2 UserName        type UNICODE_STRING,
     2 Password        type UNICODE_STRING;
 define alias PKERB_INTERACTIVE_LOGON handle KERB_INTERACTIVE_LOGON;


 define structure
   1 KERB_INTERACTIVE_UNLOCK_LOGON,
     2 Logon   type KERB_INTERACTIVE_LOGON,
     2 LogonId type LUID;
 define alias PKERB_INTERACTIVE_UNLOCK_LOGON
        handle KERB_INTERACTIVE_UNLOCK_LOGON;

 define structure
   1 KERB_SMART_CARD_LOGON,
     2 MessageType   type KERB_LOGON_SUBMIT_TYPE,
     2 Pin           type UNICODE_STRING,
     2 CspDataLength type ULONG,
     2 CspData       type PUCHAR;
 define alias PKERB_SMART_CARD_LOGON handle KERB_SMART_CARD_LOGON;

 define structure
   1 KERB_SMART_CARD_UNLOCK_LOGON,
     2 Logon type KERB_SMART_CARD_LOGON,
     2 LogonId type LUID;
 define alias PKERB_SMART_CARD_UNLOCK_LOGON
        handle KERB_SMART_CARD_UNLOCK_LOGON;

 /*                                                                   */
 /* Structure used for a ticket-only logon                            */
 /*                                                                   */

 define structure
   1 KERB_TICKET_LOGON,
     2 MessageType                type KERB_LOGON_SUBMIT_TYPE,
     2 Flags                      type ULONG,
     2 ServiceTicketLength        type ULONG,
     2 TicketGrantingTicketLength type ULONG,
     2 ServiceTicket              type PUCHAR,
       /* REQUIRED: Service ticket "host" */
     2 TicketGrantingTicket       type PUCHAR;
       /* OPTIONAL: User's encdoded in a KERB_CRED message, */
       /* encrypted with session key from service ticket    */
 define alias PKERB_TICKET_LOGON handle KERB_TICKET_LOGON;

 /*                                                                   */
 /* Flags for the ticket logon flags field                            */
 /*                                                                   */

 dcl KERB_LOGON_FLAG_ALLOW_EXPIRED_TICKET type ULONG value('1'xu);

 define structure
   1 KERB_TICKET_UNLOCK_LOGON,
     2 Logon   type KERB_TICKET_LOGON,
     2 LogonId type LUID;
 define alias PKERB_TICKET_UNLOCK_LOGON handle KERB_TICKET_UNLOCK_LOGON;

 /*                                                                   */
 /* Use the same profile structure as MSV1_0                          */
 /*                                                                   */
 define ordinal
   KERB_PROFILE_BUFFER_TYPE
     ( KerbInteractiveProfile value(2),
       KerbSmartCardProfile   value(4),
       KerbTicketProfile      value(6));
 dcl PKERB_PROFILE_BUFFER_TYPE pointer; /* to KERB_PROFILE_BUFFER_TYPE*/


 define structure
   1 KERB_INTERACTIVE_PROFILE,
     2 MessageType        type KERB_PROFILE_BUFFER_TYPE,
     2 LogonCount         type USHORT,
     2 BadPasswordCount   type USHORT,
     2 LogonTime          type LARGE_INTEGER,
     2 LogoffTime         type LARGE_INTEGER,
     2 KickOffTime        type LARGE_INTEGER,
     2 PasswordLastSet    type LARGE_INTEGER,
     2 PasswordCanChange  type LARGE_INTEGER,
     2 PasswordMustChange type LARGE_INTEGER,
     2 LogonScript        type UNICODE_STRING,
     2 HomeDirectory      type UNICODE_STRING,
     2 FullName           type UNICODE_STRING,
     2 ProfilePath        type UNICODE_STRING,
     2 HomeDirectoryDrive type UNICODE_STRING,
     2 LogonServer        type UNICODE_STRING,
     2 UserFlags          type ULONG;
 define alias PKERB_INTERACTIVE_PROFILE handle KERB_INTERACTIVE_PROFILE;


 /*                                                                   */
 /* For smart card, we return a smart card profile, which is an       */
 /* interactive profile plus a certificate                            */
 /*                                                                   */

 define structure
   1 KERB_SMART_CARD_PROFILE,
     2 Profile         type KERB_INTERACTIVE_PROFILE,
     2 CertificateSize type ULONG,
     2 CertificateData type PUCHAR;
 define alias PKERB_SMART_CARD_PROFILE handle KERB_SMART_CARD_PROFILE;


 /*                                                                   */
 /* For a ticket logon profile, we return the session key from the    */
 /* ticket                                                            */
 /*                                                                   */


 define structure
   1 KERB_CRYPTO_KEY,
     2 KeyType type LONG,
     2 Length  type ULONG,
     2 Value   type PUCHAR;
 define alias PKERB_CRYPTO_KEY handle KERB_CRYPTO_KEY;

 define structure
   1 KERB_TICKET_PROFILE,
     2 Profile type KERB_INTERACTIVE_PROFILE,
     2 SessionKey type KERB_CRYPTO_KEY;
 define alias PKERB_TICKET_PROFILE handle KERB_TICKET_PROFILE;




 define ordinal
   KERB_PROTOCOL_MESSAGE_TYPE
     ( KerbDebugRequestMessage          value(0),
       KerbQueryTicketCacheMessage,
       KerbChangeMachinePasswordMessage,
       KerbVerifyPacMessage,
       KerbRetrieveTicketMessage,
       KerbUpdateAddressesMessage,
       KerbPurgeTicketCacheMessage,
       KerbChangePasswordMessage,
       KerbRetrieveEncodedTicketMessage,
       KerbDecryptDataMessage,
       KerbAddBindingCacheEntryMessage,
       KerbSetPasswordMessage);
 dcl PKERB_PROTOCOL_MESSAGE_TYPE pointer;
                                     /* to KERB_PROTOCOL_MESSAGE_TYPE */


 /*                                                                   */
 /* Used both for retrieving tickets and for querying ticket cache    */
 /*                                                                   */

 define structure
   1 KERB_QUERY_TKT_CACHE_REQUEST,
     2 MessageType type KERB_PROTOCOL_MESSAGE_TYPE,
     2 LogonId     type LUID;
 define alias PKERB_QUERY_TKT_CACHE_REQUEST
        handle KERB_QUERY_TKT_CACHE_REQUEST;


 define structure
   1 KERB_TICKET_CACHE_INFO,
     2 ServerName     type UNICODE_STRING,
     2 RealmName      type UNICODE_STRING,
     2 StartTime      type LARGE_INTEGER,
     2 EndTime        type LARGE_INTEGER,
     2 RenewTime      type LARGE_INTEGER,
     2 EncryptionType type LONG,
     2 TicketFlags    type ULONG;
 define alias PKERB_TICKET_CACHE_INFO handle KERB_TICKET_CACHE_INFO;


 define structure
   1 KERB_QUERY_TKT_CACHE_RESPONSE,
     2 MessageType             type KERB_PROTOCOL_MESSAGE_TYPE,
     2 CountOfTickets          type ULONG,
     2 Tickets (ANYSIZE_ARRAY) type KERB_TICKET_CACHE_INFO;
 define alias PKERB_QUERY_TKT_CACHE_RESPONSE
        handle KERB_QUERY_TKT_CACHE_RESPONSE;

 /*                                                                   */
 /* Types for retrieving encoded ticket from the cache                */
 /*                                                                   */

 %dcl #_SECHANDLE_DEFINED__ char ext;
 %if #_SECHANDLE_DEFINED__ = '' %then %do;
 define structure
   1 SecHandle,
     2 dwLower type ULONG_PTR,
     2 dwUpper type ULONG_PTR;
 define alias PSecHandle handle SecHandle;

 %dcl #_SECHANDLE_DEFINED__ char ext;
 %#_SECHANDLE_DEFINED__ = 'Y';
 %end; /* __SECHANDLE_DEFINED__ */

 define structure
   1 KERB_RETRIEVE_TKT_REQUEST,
     2 MessageType       type KERB_PROTOCOL_MESSAGE_TYPE,
     2 LogonId           type LUID,
     2 TargetName        type UNICODE_STRING,
     2 TicketFlags       type ULONG,
     2 CacheOptions      type ULONG,
     2 EncryptionType    type LONG,
     2 CredentialsHandle type SecHandle;
 define alias PKERB_RETRIEVE_TKT_REQUEST
        handle KERB_RETRIEVE_TKT_REQUEST;

 dcl KERB_RETRIEVE_TICKET_DONT_USE_CACHE type ULONG value('1'xu);
 dcl KERB_RETRIEVE_TICKET_USE_CACHE_ONLY type ULONG value('2'xu);
 dcl KERB_RETRIEVE_TICKET_USE_CREDHANDLE type ULONG value('4'xu);

 /*                                                                   */
 /* Types for the information about a ticket                          */
 /*                                                                   */

 define structure
   1 KERB_EXTERNAL_NAME,
     2 NameType              type SHORT,
     2 NameCount             type USHORT,
     2 Names (ANYSIZE_ARRAY) type UNICODE_STRING;
 define alias PKERB_EXTERNAL_NAME handle KERB_EXTERNAL_NAME;


 define structure
   1 KERB_EXTERNAL_TICKET,
     2 ServiceName         type PKERB_EXTERNAL_NAME,
     2 TargetName          type PKERB_EXTERNAL_NAME,
     2 ClientName          type PKERB_EXTERNAL_NAME,
     2 DomainName          type UNICODE_STRING,
     2 TargetDomainName    type UNICODE_STRING,
     2 AltTargetDomainName type UNICODE_STRING,
     2 SessionKey          type KERB_CRYPTO_KEY,
     2 TicketFlags         type ULONG,
     2 Flags               type ULONG,
     2 KeyExpirationTime   type LARGE_INTEGER,
     2 StartTime           type LARGE_INTEGER,
     2 EndTime             type LARGE_INTEGER,
     2 RenewUntil          type LARGE_INTEGER,
     2 TimeSkew            type LARGE_INTEGER,
     2 EncodedTicketSize   type ULONG,
     2 EncodedTicket       type PUCHAR;
 define alias PKERB_EXTERNAL_TICKET handle KERB_EXTERNAL_TICKET;

 define structure
   1 KERB_RETRIEVE_TKT_RESPONSE,
    2 Ticket type KERB_EXTERNAL_TICKET;

 define alias PKERB_RETRIEVE_TKT_RESPONSE
        handle KERB_RETRIEVE_TKT_RESPONSE;

 /*                                                                   */
 /* Used to purge entries from the ticket cache                       */
 /*                                                                   */

 define structure
   1 KERB_PURGE_TKT_CACHE_REQUEST,
     2 MessageType type KERB_PROTOCOL_MESSAGE_TYPE,
     2 LogonId     type LUID,
     2 ServerName  type UNICODE_STRING,
     2 RealmName   type UNICODE_STRING;
 define alias PKERB_PURGE_TKT_CACHE_REQUEST
        handle KERB_PURGE_TKT_CACHE_REQUEST;



 /*                                                                   */
 /* KerbChangePassword                                                */
 /*                                                                   */
 /* KerbChangePassword changes the password on the KDC account plus   */
 /*  the password cache and logon credentials if applicable.          */
 /*                                                                   */
 /*                                                                   */

 define structure
   1 KERB_CHANGEPASSWORD_REQUEST,
     2 MessageType   type KERB_PROTOCOL_MESSAGE_TYPE,
     2 DomainName    type UNICODE_STRING,
     2 AccountName   type UNICODE_STRING,
     2 OldPassword   type UNICODE_STRING,
     2 NewPassword   type UNICODE_STRING,
     2 Impersonating type BOOLEAN;
 define alias PKERB_CHANGEPASSWORD_REQUEST
        handle KERB_CHANGEPASSWORD_REQUEST;

 /*                                                                   */
 /* KerbSetPassword                                                   */
 /*                                                                   */
 /* KerbSetPassword changes the password on the KDC account plus      */
 /*  the password cache and logon credentials if applicable.          */
 /*                                                                   */


 define structure
   1 KERB_SETPASSWORD_REQUEST,
     2 MessageType       type KERB_PROTOCOL_MESSAGE_TYPE,
     2 LogonId           type LUID,
     2 CredentialsHandle type SecHandle,
     2 Flags             type ULONG,
     2 DomainName        type UNICODE_STRING,
     2 AccountName       type UNICODE_STRING,
     2 Password          type UNICODE_STRING;
 define alias PKERB_SETPASSWORD_REQUEST handle KERB_SETPASSWORD_REQUEST;

 dcl KERB_SETPASS_USE_LOGONID    fixed bin(31) value(1);
 dcl KERB_SETPASS_USE_CREDHANDLE fixed bin(31) value(2);


 define structure
   1 KERB_DECRYPT_REQUEST,
     2 MessageType       type KERB_PROTOCOL_MESSAGE_TYPE,
     2 LogonId           type LUID,
     2 Flags             type ULONG,
     2 CryptoType        type LONG,
     2 KeyUsage          type LONG,
     2 Key               type KERB_CRYPTO_KEY,/* optional */
     2 EncryptedDataSize type ULONG,
     2 InitialVectorSize type ULONG,
     2 InitialVector     type PUCHAR,
     2 EncryptedData     type PUCHAR;
 define alias PKERB_DECRYPT_REQUEST handle KERB_DECRYPT_REQUEST;

 /*                                                                   */
 /* If set, use the primary key from the current logon session of the */
 /* one provided in the LogonId field.                                */
 /* Otherwise, use the Key in the KERB_DECRYPT_MESSAGE.               */

 dcl KERB_DECRYPT_FLAG_DEFAULT_KEY fixed bin(31) value('00000001'xn);


 define structure
   1 KERB_DECRYPT_RESPONSE,
     2 DecryptedData (ANYSIZE_ARRAY) type UCHAR;
 define alias PKERB_DECRYPT_RESPONSE handle KERB_DECRYPT_RESPONSE;


 /*                                                                   */
 /* Request structure for adding a binding cache entry. TCB privilege */
 /* is required for this operation.                                   */
 /*                                                                   */

 define structure
   1 KERB_ADD_BINDING_CACHE_ENTRY_REQUEST,
     2 MessageType type KERB_PROTOCOL_MESSAGE_TYPE,
     2 RealmName   type UNICODE_STRING,
     2 KdcAddress  type UNICODE_STRING,
     2 AddressType type ULONG; /* from dsgetdc.cpy */

 define alias PKERB_ADD_BINDING_CACHE_ENTRY_REQUEST
        handle KERB_ADD_BINDING_CACHE_ENTRY_REQUEST;

 %end; /* _NTSECAPI_ */
