 /********************************************************************/
 /*                                                                  */
 /*  PL/I Include file: SQLDA_T.CPY                                  */
 /*                                                                  */
 /*    This file is installed in the \IBMPLIW\INCLUDE\ directory.    */
 /*                                                                  */
 /*                                                                  */
 /*  NAME - SQLDA_T.CPY                                              */
 /*                                                                  */
 /*  DESCRIPTION                                                     */
 /*                                                                  */
 /*    External Include file for SQLDA using user-defined types      */
 /*    and structures.                                               */
 /*                                                                  */
 /*    (C) Copyright IBM Corp. 1994, 1997. All Rights Reserved.      */
 /*    US Government Users Restricted Rights-- Use, duplication or   */
 /*    disclosure restricted by GSA ADP Schedule Contract with       */
 /*    IBM Corp.                                                     */
 /*  Licensed Materials - Property of IBM                            */
 /*                                                                  */
 /*  Operating System = Windows                                      */
 /*                                                                  */
 /*                                                                  */
 /*  DISCLAIMER OF WARRANTIES                                        */
 /*    The following enclosed code is sample code created by IBM   */
 /*    Corporation. This sample code is not part of any standard     */
 /*    IBM product and is provided to you solely for the purpose of  */
 /*    assisting you in the development of your applications.  The   */
 /*    code is provided "AS IS", without warranty of any kind.       */
 /*    IBM shall not be liable for any damages arising out of your   */
 /*    use of the sample code, even if IBM has been advised of the   */
 /*    possibility of such damages.                                  */
 /*                                                                  */
 /********************************************************************/

 %dcl #SQLDASIZE char ext;
 %if #SQLDASIZE = '' %then %do;
 %#SQLDASIZE = 'Y';

 %dcl SQL_H_SQLSYSTM char ext;
 %if SQL_H_SQLSYSTM = '' %then %do;
 %include sqlsystm;     /* System-Specific Include                   */
 %end;

 /* SQLDA: SQL Descriptor Area - Variable descriptor*/

 define structure
   1 sqlname,                   /* Variable Name                    */
     2 length    fixed bin(15), /* Name length [1..30]              */
     2 data      char(30);      /* Variable or Column name          */


 define structure
   1 sqldistinct_type,          /* name of distinct type            */
     2 length    fixed bin(15), /* Name length [1..27]              */
     2 data      char(27),      /* Name of distince type            */
     2 reserved1 char(3) ;      /* reserved                         */


 define structure
   1 sqlvar,                    /* Variable Description             */
     2 sqltype   fixed bin(15), /* Variable data type               */
     2 sqllen    fixed bin(15), /* Variable data length             */
     2 sqldata   pointer,       /* Pointer to variable data value   */
     2 sqlind    pointer,       /* Pointer to Null indicator        */
     2 sqlname   type sqlname;  /* Variable name                    */


 define structure
   1 sqlda,
     2 sqldaid   char(8),       /* Eye catcher = 'SQLDA '           */
     /***************************************************************/
     /* The 7th byte has special meaning. If it is a '2', this means*/
     /* there are twice as many sqlvars as there are host variables */
     /* or columns.  This is typically used when Large Objects      */
     /* (LOBs) or User Defined Types (UDFs) are present in the      */
     /* SQLDA. The first N entries use the sqlvar structure and the */
     /* second N entries use the sqlvar2 structure.                 */
     /***************************************************************/
     2 sqldabc   fixed bin(31),   /* SQLDA size in bytes=16+44*SQLN */
     2 sqln      fixed bin(15),   /* Number of SQLVAR elements      */
     2 sqld      fixed bin(15),   /* # of columns or host vars.     */
     2 sqlvar(1) type sqlvar;     /* SQLVAR element (REFER)         */

 /* macro for allocating SQLDA */
 %SQLDASIZE: proc (n) returns (char);
   dcl n char;
   return ('(size (: SQLDA :) + ' ||
            '((' || n || ') - 1) * size (: SQLVAR :) )');
 %end SQLDASIZE;

 /********************************************************************/
 /* Because we may need to support 8 byte lengths in the future,     */
 /* sql8bytelen is defined using 8 bytes.  Currently, however, we    */
 /* have only four byte integers.                                    */
 /********************************************************************/
 define structure
   1 sql8bytelen union,
     2 reserved1(2) fixed bin(31), /* rsvd for future 8 byte lengths */
     2 sqllonglen   fixed bin(31); /* this is what is currently used */

 define structure
   1 sql4bytelen union,
     2 reserved1    char(4),       /* reserved                       */
     2 sqllonglen   fixed bin(31); /* this is what is currently used */

 /********************************************************************/
 /* The sqlvar2 structure maps the second "N" sqlvar entries.  The   */
 /* second "N" entries are used to hold the length of LOB columns    */
 /* and host variables.  The second "N" entries may also be used to  */
 /* hold the SQLDATALEN field for LOB columns on a FETCH USING       */
 /* DESCRIPTOR request.                                              */
 /*                                                                  */
 /* To set or retrieve these fields, use the macros provided below.  */
 /********************************************************************/

 define structure
   1 sqlvar2,            /* Variable Description                     */
     2 len   type sql4bytelen,/* Eight byte length. 4 bytes used now */
     2 reserve2 char(3), /* Reserved                                 */
     2 sqlflag4 char(1), /* Var is reference to a row type if 0x01   */
     2 sqldatalen pointer,  /* Pointer to four (4) byte length buffer*/
                            /* This may be used to hold the length   */
                            /* for lob data types.                   */
     2 sqldatatype_name type sqldistinct_type; /* distinct type name.*/

 /*********************************************************************/
 /* Flag values for sqlflag4 field of sqlvar2.                        */
 /*********************************************************************/

 dcl SQLFLAG4_BASE   char value ('00'x);
                                /* Type is base type or distinct type */
 dcl SQLFLAG4_REF    char value ('01'x); /* Reference type            */
 dcl SQLFLAG4_STRUCT char value ('12'x); /* Structured type           */

 /*********************************************************************/
 /* Macros for using the sqlvar2 fields.                              */
 /*********************************************************************/

 /*********************************************************************/
 /* A '2' in the 7th byte of sqldaid indicates a doubled amount of    */
 /* sqlvars.                                                          */
 /*********************************************************************/
 dcl SQLDOUBLED char value('2');
 dcl SQLSINGLED char value(' ');

 /*********************************************************************/
 /* GETSQLDOUBLED(daptr) will return 1 if the SQLDA pointed to by     */
 /* daptr has been doubled, or 0 if it has not been doubled.          */
 /*********************************************************************/
 GETSQLDOUBLED: proc (daptr)
                returns (fixed bin(31) byvalue)
                options (inline byvalue) internal;
   dcl daptr handle sqlda;

   return (substr (daptr => sqldaid, 7, 1) = SQLDOUBLED);
 end GETSQLDOUBLED;

 /********************************************************************/
 /* SETSQLDOUBLED(daptr, SQLDOUBLED) will make the 7th byte of       */
 /* sqldaid to be a '2'.                                             */
 /* SETSQLDOUBLED(daptr, SQLSINGLE)  will make the 7th byte of       */
 /* sqldaid to be a ' '.                                             */
 /********************************************************************/
 SETSQLDOUBLED: proc (daptr, newvalue)
                options (inline byvalue) internal;
   dcl daptr    handle sqlda,
       newvalue char(1);

   substr (daptr => sqldaid, 7, 1) = newvalue;
 end SETSQLDOUBLED;

 /********************************************************************/
 /* GETSQLDALONGLEN(daptr,n) will return the data length of the nth  */
 /* entry in the sqlda pointed to by daptr. Use this only if the     */
 /* sqlda was doubled and the nth SQLVAR entry has a LOB datatype.   */
 /********************************************************************/
 (nosubscriptrange):
 GETSQLDALONGLEN: proc (daptr,n)
                  returns (fixed bin(31) byvalue)
                  options (inline byvalue) internal;

   dcl daptr  handle sqlda,
       n      fixed bin(32) unsigned;
   dcl p2     handle sqlvar2;
   dcl handle builtin;

   p2 = handle (daptr => sqlvar (n + daptr => sqld));

   return (p2 => len.sqllonglen);
 end GETSQLDALONGLEN;

 /********************************************************************/
 /* SETSQLDALONGLEN(daptr,n,len) will set the sqllonglen field of    */
 /* the sqlda pointed to by daptr to len for the nth entry. Use this */
 /* only if the sqlda was doubled and the nth SQLVAR entry has a LOB */
 /* datatype.                                                        */
 /********************************************************************/

 (nosubscriptrange):
 SETSQLDALONGLEN: proc (daptr, n, length)
                  options (inline byvalue) internal;
   dcl daptr  handle sqlda,
       n      fixed bin(32) unsigned,
       length fixed bin (31);
   dcl p2     handle sqlvar2;
   dcl handle builtin;

   p2 = handle (daptr => sqlvar (n + daptr => sqld));
   p2 => len.sqllonglen = length;
 end SETSQLDALONGLEN;

 /********************************************************************/
 /* GETSQLDALENPTR(daptr,n) will return a pointer to the data length */
 /* for the nth entry in the sqlda pointed to by daptr.  Unlike the  */
 /* inline value (union sql8bytelen len), which is 8 bytes, the      */
 /* sqldatalen pointer field returns a pointer to a long (4 byte)    */
 /* integer.                                                         */
 /* If the SQLDATALEN pointer is zero, a NULL pointer will be        */
 /* returned.                                                        */
 /*                                                                  */
 /* NOTE: Use this only if the sqlda has been doubled.               */
 /********************************************************************/

 (nosubscriptrange):
 GETSQLDALENPTR: proc (daptr, n)
                 returns (pointer byvalue)
                 options (inline byvalue) internal;
   dcl daptr  handle sqlda,
       n      fixed bin(32) unsigned;
   dcl p2     handle sqlvar2;
   dcl handle builtin;

   p2 = handle (daptr => sqlvar (n + daptr => sqld));

   return (p2 => sqldatalen);
 end GETSQLDALENPTR;

 /********************************************************************/
 /* SETSQLDALENPTR(daptr,n,ptr) will set a pointer to the data length*/
 /* for the nth entry in the sqlda pointed to by daptr.              */
 /* Use this only if the sqlda has been doubled.                     */
 /********************************************************************/

 (nosubscriptrange):
 SETSQLDALENPTR: proc (daptr, n, ptr)
                 options (inline byvalue) internal;
   dcl daptr  handle sqlda,
       n      fixed bin(32) unsigned,
       ptr    pointer;
   dcl p2     handle sqlvar2;
   dcl handle builtin;

   p2 = handle (daptr => sqlvar (n + daptr => sqld));
   p2 => sqldatalen = ptr;
 end SETSQLDALENPTR;


 /*********************************************************************/
 /* GETSQLDAFLAG4(daptr,n) will return the REFERENCE type character   */
 /* flag from the nth entry in the sqlda pointed to by daptr.         */
 /* Use this only if the sqlda was doubled.                           */
 /*********************************************************************/
 (nosubscriptrange):
 GETSQLDAFLAG4: proc (daptr,n)
                returns (char (1) byvalue)
                options (inline byvalue) internal;
   dcl daptr  handle sqlda,
       n      fixed bin (31);
   dcl p2     handle sqlvar2;
   dcl handle builtin;

   p2 = handle (daptr => sqlvar (n + daptr => sqld));
   return (p2 => sqlflag4);
 end GETSQLDAFLAG4;

 /*********************************************************************/
 /* SETSQLDAFLAG4(daptr,n,flag) will return the REFERENCE type char   */
 /* flag from the nth entry in the sqlda pointed to by daptr to flag. */
 /* Use this only if the sqlda was doubled.                           */
 /*********************************************************************/

 (nosubscriptrange):
 SETSQLDAFLAG4: proc (daptr, n, flag)
                options (inline byvalue) internal;
   dcl daptr  handle sqlda,
       n      fixed bin (31),
       flag   char (1);
   dcl p2     handle sqlvar2;
   dcl handle builtin;

   p2 = handle (daptr => sqlvar (n + daptr => sqld));
   p2 => sqlflag4 = flag;
 end SETSQLDAFLAG4;

 %end; /* #SQLDASIZE */
