 /*********************************************************************/
 /*                                                                   */
 /* Source File Name = SQLXA.CPY                                      */
 /*                                                                   */
 /* (C) COPYRIGHT International Business Machines Corp. 1993, 1995    */
 /* All Rights Reserved                                               */
 /* Licensed Materials - Property of IBM                              */
 /*                                                                   */
 /* US Government Users Restricted Rights - Use, duplication or       */
 /* disclosure restricted by GSA ADP Schedule Contract with IBM Corp. */
 /*                                                                   */
 /* Function = Include File defining:                                 */
 /*              XA Interface - Constants                             */
 /*              XA Interface - Data Structures                       */
 /*              XA Interface - Function Prototypes                   */
 /*                                                                   */
 /* Operating System = Common PL/I Include File                       */
 /*                                                                   */
 /*********************************************************************/

 %dcl SQL_H_SQLXA char ext;
 %if SQL_H_SQLXA = '' %then %do;
 %SQL_H_SQLXA = 'Y'; /* Permit duplicate Includes */

 %dcl SQL_H_SQLSYSTM char ext;
 %if SQL_H_SQLSYSTM = '' %then %do;
 %include sqlsystm;
 %end;
 %dcl SQL_H_DB2APIDF char ext;
 %if SQL_H_DB2APIDF = '' %then %do;
 %include db2apidf;
 %end;



 /* Define an alias for the XID structure defined by X/Open.       */
 /* This avoids conflicts with the XID defined by X11 (Xwindows).  */

 dcl SQLXA_XIDDATASIZE   fixed bin(31) value(128); /* size in bytes */

 dcl SQLXA_XID_FORMATID  fixed bin(31) value('53514C20'xn); /* "SQL " */
 dcl SQLXA_XID_BQLEN     fixed bin(31) value(0);
 dcl SQLXA_XID_APPLIDLEN fixed bin(31) value(25);
 dcl SQLXA_XID_SEQNUM    fixed bin(31) value(4);
 dcl SQLXA_XID_GTLEN     fixed bin(31)
     value(SQL_DBNAME_SZ+1+SQLXA_XID_APPLIDLEN+SQLXA_XID_SEQNUM+1);

 dcl SQLXA_XID_TSLEN     fixed bin(31) value(4);
 dcl SQLXA_XID_LOGIDLEN  fixed bin(31) value(4);
 dcl SQLXA_XID_LOGIDPOS  fixed bin(31)
     value(SQL_DBNAME_SZ+1+SQLXA_XID_TSLEN);
 dcl SQLXA_XID_TIDPOS    fixed bin(31)
     value(SQLXA_XID_LOGIDPOS+SQLXA_XID_LOGIDLEN);
 dcl SQLXA_XID_TIDLEN    fixed bin(31) value(6);

 define structure
   1 sqlxa_xid_t,
     2 formatID     fixed bin(31),/* format identifier */
     2 gtrid_length fixed bin(31),/* value from 1 through 64 */
     2 bqual_length fixed bin(31),/* value from 1 through 64 */
     2 data         char(SQLXA_XIDDATASIZE);
 define alias SQLXA_XID type sqlxa_xid_t;

 /* the following macro can generate a NULLXID */
 SQLXA_NULLXID: proc (x) options(inline) internal;
   dcl x type sqlxa_xid_t byaddr;
   dcl low builtin;
   x.formatID     = -1;
   x.gtrid_length = -1;
   x.bqual_length = -1;
   x.data = low(SQLXA_XIDDATASIZE); /* in C: only init 1st byte */
 end SQLXA_NULLXID;

 /* provide a test for the NULLXID */
 /* note that we only test the formatID! -> a simplification */
 SQLXA_XID_IS_NULL: proc (x) returns (bit (1) aligned)
                    options (inline) internal;
   dcl x type sqlxa_xid_t byaddr nonasgn;
   return  (x.formatID = -1);
 end SQLXA_XID_IS_NULL;

 /* provide a test for Microsoft DTC */
 SQLXA_XID_IS_MSDTC: proc (x) returns (bit (1) aligned)
                     options (inline) internal;
   dcl x type sqlxa_xid_t byaddr nonasgn;
   return (x.formatID = '00445443'xn);
 end SQLXA_XID_IS_MSDTC;

 /* provide a test for Encina Server */
 SQLXA_XID_IS_ENCINA: proc (x) returns (bit (1) aligned)
                      options (inline) internal;
   dcl x type sqlxa_xid_t byaddr nonasgn;
   return (x.formatID   = 113577);
 end SQLXA_XID_IS_ENCINA;

 /* provide a test for SYSTEM_NU Server */
 SQLXA_XID_IS_SYSTEM_NU: proc (x) returns (bit (1) aligned)
                         options (inline) internal;
   dcl x type sqlxa_xid_t byaddr nonasgn;
   return (x.formatID = 33);
 end SQLXA_XID_IS_SYSTEM_NU;

 /* provide a test for DB2 coordinated transaction */
 SQLXA_XID_IS_DB2: proc (x) returns (bit (1) aligned)
                   options (inline) internal;
   dcl x type sqlxa_xid_t byaddr nonasgn;
   return (x.formatID = SQLXA_XID_FORMATID);
 end SQLXA_XID_IS_DB2;

 /* provide a test for DB2 coordinated transaction */
 SQLXA_XID_IS_DB2_DRDA: proc (x) returns (bit (1) aligned)
                        options (inline) internal;
   dcl x type sqlxa_xid_t byaddr nonasgn;
   return (x.formatID = '44524441'xn);
 end SQLXA_XID_IS_DB2_DRDA;

 /* provide XID header size ...sum of size of */
 /* formatID,gtrid_size,bqual_size            */
 dcl XID_HEADER_SIZE fixed bin (31) value (12);

 /* provide true XID size */
 XID_ACTUAL_SIZE: proc (p_xid) returns (fixed bin (31))
                               options (inline) internal;
   dcl p_xid handle sqlxa_xid_t byvalue nonasgn;
   return (XID_HEADER_SIZE + p_xid => gtrid_length
                           + p_xid => bqual_length);

 end XID_ACTUAL_SIZE;

 /* Compare xidA and xidB, for equality.                           */
 /* Returns "true" if compared to NULLXID by checking the formatID */
 /* Returns "true" if xid's are the same.                          */

 XIDEQUAL: proc (xidA, xidB) returns (bit (1) aligned)
           options (inline) internal;
   dcl xidA type sqlxa_xid_t byaddr nonasgn,
       xidB type sqlxa_xid_t byaddr nonasgn;
       dcl (addr, compare) builtin;
   if xidA.formatID = -1 & xidB.formatID = -1 then
     return ('1'b);
   if xidA.formatID = xidB.formatID
    & xidA.gtrid_length = xidB.gtrid_length
    & xidA.bqual_length = xidB.bqual_length
    & compare (addr (xidA.data), addr(xidB.data),
               xidA.gtrid_length + xidA.bqual_length) = 0 then
     return ('1'b);
   return ('0'b);
 end XIDEQUAL;

 /* Copy xidB into xidA */
 /* -> a macro is not really needed here, since the PL/I language */
 /* supports structure assignments                                */
 XIDCPY: proc (xidA, xidB) options (inline) internal;
   dcl xidA type sqlxa_xid_t byaddr,
       xidB type sqlxa_xid_t byaddr nonasgn;
   xidA = xidB;
 end XIDCPY;


 /* define a TRUE/FALSE pair for boolean fields in XA structures */
 dcl SQLXA_TRUE  fixed bin(8) unsigned value(1);
 dcl SQLXA_FALSE fixed bin(8) unsigned value(0);

 /* define all possible states of transactions returned by "recover" */
 dcl SQLXA_TS_PREP char value('P'); /* prepared                       */
 dcl SQLXA_TS_HCOM char value('X'); /* heuristically committed        */
 dcl SQLXA_TS_HROL char value('Y'); /* heuristically rolled back      */
 dcl SQLXA_TS_END  char value('E'); /* idle -> xa_end has been issued */
 dcl SQLXA_TS_MACK char value('M'); /* missing commit acknowledgement */

 /* possible indoubt states in Parallel Edition ONLY */
 /* other than the above                             */
 dcl SQLXA_TS_PHC   char value('A');/* prepared and heur. committed */
 dcl SQLXA_TS_PHR   char value('B');/* prepared and heur. rolled back */
 dcl SQLXA_TS_HCHR  char value('C');
     /* heur. committed and heur. rolled back */
 dcl SQLXA_TS_PHCHR char value('D');
     /* prepared, heur. committed and heur. rolled back */

 dcl SQLXA_EXE_ALL_NODES fixed bin(31) value(1);
     /* execute the request on all the nodes in MPP environment */
 dcl SQLXA_EXE_THIS_NODE fixed bin(31) value(0);
     /* execute the request on the node it is issued from */

 /* various transaction originators */
 dcl SQLXA_ORIG_XA fixed bin(8) unsigned value(1);
     /* transaction originated by XA */
 dcl SQLXA_ORIG_PE fixed bin(8) unsigned value(0);
     /* transaction originated by DB2 in MPP environment*/

 /* The following structure is used to describe the current state of  */
 /* an indoubt (ie "prepared") or heuristically completed transaction.*/
 /*                                                                   */
 /* We need to beef up the explanations of the structure elements,    */
 /* so that they can appear in the user doc -> CLP will be providing  */
 /* the info basically verbatim to the user.                          */
 /*                                                                   */
 /* The char string areas will always be right padded with blanks.    */

 dcl SQLXA_DBNAME_SZ     fixed bin(31) value(8);
 dcl SQLXA_APPLID_SZ     fixed bin(31) value(32);
 dcl SQLXA_SEQ_SZ        fixed bin(31) value(4);
 dcl SQLXA_OLD_USERID_SZ fixed bin(31) value(8);
 dcl SQLXA_USERID_SZ     fixed bin(31) value(SQL_USERID_SZ);
 dcl SQLXA_PASSWD_SZ     fixed bin(31) value(18); /* @d171394iis */

 define structure
   1 sqlxa_recover_t,
     2 timestamp      fixed bin(32) unsigned,
     2 xid            type SQLXA_XID,
     2 dbalias        char(SQLXA_DBNAME_SZ),
     2 applid         char(SQLXA_APPLID_SZ),
     2 sequence_no    char(SQLXA_SEQ_SZ),
     2 auth_id        char(SQLXA_USERID_SZ),
     2 log_full       fixed bin(8) unsigned, /* SQLXA_TRUE/FALSE */
     2 connected      fixed bin(8) unsigned, /* SQLXA_TRUE/FALSE */
     2 indoubt_status char(1),               /* SQLXA_TS_xxx     */
     2 originator     fixed bin(8) unsigned, /* SQLXA_ORIG_PE/XA */
     2 reserved       char(8);               /* set to zeroes    */
 define alias SQLXA_RECOVER type sqlxa_recover_t;

 /**** reason codes for the SQL_RC_W997 general XA warning code */
 dcl SQLXAER_REASON_DEADLOCK               fixed bin(31) value(1);
     /* returned when XA_END clears a deadlock */
 dcl SQLXAER_REASON_RDONLYCOM              fixed bin(31) value(35);
     /* trans read-only and has been committed */

 /**** reason codes for the SQL_RC_E998 general XA error sqlcode */
 dcl SQLXAER_REASON_ASYNC                  fixed bin(31) value(1);
     /* asynch operation already outstanding     */
 dcl SQLXAER_REASON_RMERR                  fixed bin(31) value(2);
     /* an RM error occurred                     */
 dcl SQLXAER_REASON_NOTA                   fixed bin(31) value(3);
     /* XID not recognized                       */
 dcl SQLXAER_REASON_INVAL                  fixed bin(31) value(4);
     /* invalid parameters                       */
 dcl SQLXAER_REASON_PROTO                  fixed bin(31) value(5);
     /* routine called in improper context       */
 dcl SQLXAER_REASON_RMFAIL                 fixed bin(31) value(6);
     /* RM unavailable                           */
 dcl SQLXAER_REASON_DUPID                  fixed bin(31) value(7);
     /* the XID already exists                   */
 dcl SQLXAER_REASON_OUTSIDE                fixed bin(31) value(8);
     /* RM doing work outside global transaction */
 dcl SQLXAER_REASON_AXREG                  fixed bin(31) value(9);
     /* axreg failed                             */
 dcl SQLXAER_REASON_1LUW                   fixed bin(31) value(10);
     /* trying to start new trans while suspended*/
 dcl SQLXAER_REASON_NOJOIN                 fixed bin(31) value(11);
     /* can't joing work of existing transaction */
 dcl SQLXAER_REASON_AXUNREG                fixed bin(31) value(12);
     /* axunreg failed                           */
 dcl SQLXAER_REASON_BADAX                  fixed bin(31) value(13);
     /* ax interface failure (unresolved symbol) */
 dcl SQLXAER_REASON_DTCXATMDOWN            fixed bin(31) value(14);
     /* Enlist TM with DTC has failed        */
 dcl SQLXAER_REASON_DTCNOTRANSACTION       fixed bin(31) value(15);
     /* AXREG with DTC has failed        */
 dcl SQLXAER_REASON_NOHEUR                 fixed bin(31) value(35);
     /* heuristic operaion invalid for non-XA db */
 dcl SQLXAER_REASON_BADXID                 fixed bin(31) value(36);
     /* XID not known by the RM                 */
 dcl SQLXAER_REASON_HEURCOM                fixed bin(31) value(37);
     /* trans already heuristically commited     */
 dcl SQLXAER_REASON_HEURROL                fixed bin(31) value(38);
     /* trans already heuristically rolled back  */
 dcl SQLXAER_REASON_NOTINDT                fixed bin(31) value(39);
     /* not an indoubt transaction               */
 dcl SQLXAER_REASON_NOTSUPP                fixed bin(31) value(40);
     /* operation not supported                  */
 dcl SQLXAER_REASON_HEURFAL                fixed bin(31) value(41);
     /* heuristic commit fails because of node   */
     /* failure                                  */

 /**** subcodes for the SQL_RC_E998 general XA error sqlcode */
 /* reason code 4 (SQLXAER_REASON_INVAL)                     */
 dcl SQLXAER_SUBCODE_XAINFO_INVALID        fixed bin(31) value(1);
 dcl SQLXAER_SUBCODE_DBNAME_TOO_LONG       fixed bin(31) value(2);
 dcl SQLXAER_SUBCODE_USERID_TOO_LONG       fixed bin(31) value(3);
 dcl SQLXAER_SUBCODE_PASSWD_TOO_LONG       fixed bin(31) value(4);
 dcl SQLXAER_SUBCODE_USERID_NO_PASSWD      fixed bin(31) value(5);
 dcl SQLXAER_SUBCODE_PASSWD_NO_USERID      fixed bin(31) value(6);
 dcl SQLXAER_SUBCODE_TOO_MANY_PARMS        fixed bin(31) value(7);
 dcl SQLXAER_SUBCODE_RMID_NOT_MATCH_DBNAME fixed bin(31) value(8);
 dcl SQLXAER_SUBCODE_DBNAME_NOT_SPECIFIED  fixed bin(31) value(9);
 dcl SQLXAER_SUBCODE_INVALID_EXE_TYPE      fixed bin(31) value(10);
 dcl SQLXAER_SUBCODE_INVALID_TMFLAGS       fixed bin(31) value(14);

 /**** subcodes for the SQL_RC_E998 general XA error sqlcode */
 /* reason code 9 (SQLXAER_REASON_AXREG)                     */
 dcl SQLXAER_SUBCODE_JOINING_XID_NOT_FOUND fixed bin(31) value(1);
 dcl SQLXAER_SUBCODE_AXLIB_LOAD_FAIL       fixed bin(31) value(2);
 dcl SQLXAER_SUBCODE_TP_MON_NAME_AXLIB_NOT_FOUND
                                           fixed bin(31) value(3);

 /**** subcodes for the SQL_RC_E998 general XA error sqlcode */
 /* reason code 13 (SQLXAER_REASON_BADAX)                    */
 dcl SQLXAER_SUBCODE_AX_REG_NOT_FOUND      fixed bin(31) value(1);
 dcl SQLXAER_SUBCODE_AX_UNREG_NOT_FOUND    fixed bin(31) value(2);

 /**** subcodes for the SQL_RC_E998 general XA error sqlcode */
 /* reason code ??                                           */
 /* for SQLXAER_SUBCODE_INVALID_EXE_TYPE, see above: reason code 4 */
 dcl SQLXAER_SUBCODE_SUSPEND_CURSOR        fixed bin(31) value(11);
 dcl SQLXAER_SUBCODE_AX_LIB                fixed bin(31) value(12);
 dcl SQLXAER_SUBCODE_TP_MON                fixed bin(31) value(13);

 /**** reason codes for the SQL_RC_E30090 general XA error code */
 dcl SQLXAER_REASON_READONLY               fixed bin(31) value(1);
     /* update issued against readonly database */
 dcl SQLXAER_REASON_BADAPI                 fixed bin(31) value(2);
     /* This API not allowed                    */
 dcl SQLXAER_REASON_CANTHOLD               fixed bin(31) value(3);
     /* HELD CURSOR not allowed                 */


 /*** prototypes for the heuristic API's in sqlxapi.cpy *************/

 dcl sqlxphqr                 /* query existing indoubt transactions  */
     entry (fixed bin(31),
            handle SQLXA_RECOVER byaddr,
            fixed bin(31) byaddr,
            pointer)                    /* SQLCA */
     returns (fixed bin(31) byvalue optional)
     SQL_API_FN external('sqlxphqr');

 dcl sqlxphcm                 /* heuristically commit a transaction   */
     entry (fixed bin(31),
            type SQLXA_XID byaddr,
            pointer)                    /* SQLCA */
     returns (fixed bin(31) byvalue optional)
     SQL_API_FN external('sqlxphcm');

 dcl sqlxphrl                 /* heuristically rollback a transaction */
     entry (fixed bin(31),
            type SQLXA_XID byaddr,
            pointer)                    /* SQLCA */
     returns (fixed bin(31) byvalue optional)
     SQL_API_FN external('sqlxphrl');

 dcl sqlxhfrg                 /* heuristically forget a transaction   */
     entry (type SQLXA_XID byaddr,
            pointer)                    /* SQLCA */
     returns (fixed bin(31) byvalue optional)
     SQL_API_FN external('sqlxhfrg');

 /*********************************************************************/
 /* List existing indoubt transactions                                */
 /*********************************************************************/
 define structure
   1 db2XaRecoverStruct,
     2 timestamp    type sqluint32, /* Time when indoubt state entered*/
     2 xid          type SQLXA_XID, /* XA Transaction Identifier */
     2 dbalias      char (SQLXA_DBNAME_SZ), /* Database alias */
     2 applid       char (SQLXA_APPLID_SZ), /* Application Identifier */
     2 sequence_noc char (SQLXA_SEQ_SZ),    /* Sequence Number */
     2 auth_id      char (SQLXA_USERID_SZ), /* Authorisation ID  */
     2 log_full       char (1),         /* Log full         */
     2 connected      char (1),         /* Is application connected ? */
     2 indoubt_status char (1),         /* Indoubt status  */
     2 originator     char (1),         /* XA or DB2 EEE transaction */
     2 reserved       char (8);         /* Reserved         */


 /* List Indoubt Transaction API struct */
 define structure
   1 db2XaListIndTransStruct,
   2 piIndoubtData handle db2XaRecoverStruct,
                                       /* Indoubt Data buffer pointer */
   2 iIndoubtDataLen   type db2Uint32, /* Indoubt Data buffer length */
   2 oNumIndoubtsReturned type db2Uint32,
                                       /* Number of indoubts returned */
   2 oNumIndoubtsTotal type db2Uint32, /* Total number of indoubts  */
   2 oReqBufferLen     type db2Uint32; /* Required buffer length    */


 /* db2XaListIndTrans - API */
 dcl db2XaListIndTrans
     entry (type db2Uint32,      /* Database version number           */
            pointer,             /* Input parameters                  */
            pointer)             /* SQLCA                             */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2XaListIndTrans');

 /*********************************************************************/
 /* Get Information for Resource Manager(rmid)                        */
 /*********************************************************************/
 define structure
   1 db2XaGetInfoStruct,
     2 iRmid      type db2int32,     /* rmid to get information for   */
     2 oLastSqlca type SQLCA;        /* sqlca for last XA API call    */


 dcl db2XaGetInfo
     entry (type db2Uint32,      /* Database version number           */
            pointer,             /* Input parameters                  */
            type sqlca byaddr)   /* SQLCA                             */
     returns (type SQL_API_RC byvalue optional)
     SQL_API_FN external('db2XaGetInfo');


 /*** map old api to new ones ***/
 sqlxhqry: proc (_ppIndoubtData, _pNumIndoubts, _pSqlca)
           returns (fixed bin(31) byvalue optional)
           options (inline) internal;
   dcl _ppIndoubtData handle SQLXA_RECOVER byaddr,
       _pNumIndoubts  fixed bin(31) byaddr,
       _pSqlca        pointer;          /* SQLCA */
   return (sqlxphqr (SQLXA_EXE_THIS_NODE,
                     _ppIndoubtData, _pNumIndoubts, _pSqlca));
 end sqlxhqry;

 sqlxhcom: proc (_pTransId, _pSqlca)
           returns (fixed bin(31) byvalue optional)
           options (inline) internal;
   dcl _pTransId      type SQLXA_XID byaddr,
       _pSqlca        pointer;          /* SQLCA */
   return (sqlxphcm (SQLXA_EXE_THIS_NODE, _pTransId, _pSqlca));
 end sqlxhcom;

 sqlxhrol: proc (_pTransId, _pSqlca)
           returns (fixed bin(31) byvalue optional)
           options (inline) internal;
   dcl _pTransId      type SQLXA_XID byaddr,
       _pSqlca        pointer;          /* SQLCA */
   return (sqlxphrl (SQLXA_EXE_THIS_NODE, _pTransId, _pSqlca));
 end sqlxhrol;

 /* The following structure is used to describe the current state */
 /* of a DRDA indoubt transaction.                                */

 dcl SQLCSPQY_DBNAME_SZ  fixed bin(31) value(8);
 dcl SQLCSPQY_LUWID_SZ   fixed bin(31) value(35);
 dcl SQLCSPQY_LUNAME_SZ  fixed bin(31) value(17);
 dcl SQLCSPQY_APPLID_SZ  fixed bin(31) value(32);

 dcl SQLCSPQY_AR         char value('R');
 dcl SQLCSPQY_AS         char value('S');

 dcl SQLCSPQY_STATUS_COM char value('C');
 dcl SQLCSPQY_STATUS_RBK char value('R');
 dcl SQLCSPQY_STATUS_IDB char value('I');
 dcl SQLCSPQY_STATUS_HCM char value('X');
 dcl SQLCSPQY_STATUS_HRB char value('Y');

 define structure
   1 sqlcspqy_indoubt_t,   /* heuristically forget a transaction   */
     2 xid            type SQLXA_XID,
     2 luwid          char(SQLCSPQY_LUWID_SZ),
     2 corrtok        char(SQLCSPQY_APPLID_SZ),
     2 partner        char(SQLCSPQY_LUNAME_SZ),
     2 dbname         char(SQLCSPQY_DBNAME_SZ),
     2 dbalias        char(SQLCSPQY_DBNAME_SZ),
     2 role           char(1),
     2 uow_status     char(1),
     2 partner_status char(1);
 define alias SQLCSPQY_INDOUBT type sqlcspqy_indoubt_t;

 /* function prototype for list DRDA indoubt transactions */

 dcl sqlcspqy
     entry (handle SQLCSPQY_INDOUBT byaddr,
            fixed bin(31) byaddr,
            pointer)                    /* SQLCA */
     returns (fixed bin(31) byvalue optional)
     SQL_API_FN external('sqlcspqy');

 %end; /* SQL_H_SQLXA */
